import os

import networkx as nx
import pandas as pd

from magine.data.storage import network_data_dir
from magine.logging import get_logger

log = get_logger(__name__)
_p_name = os.path.join(network_data_dir, 'trrust.p.gz')

url = 'http://www.grnpedia.org/trrust/data/trrust_rawdata.human.tsv'


def download_trrust():
    log.info("Downloading TRRUST")
    df = pd.read_csv(url,
                     names=['source', 'target', 'interactionType', 'pmid'],
                     delimiter='\t', index_col=None,
                     error_bad_lines=False, encoding='utf-8'
                     )
    int_type = 'interactionType'

    # filter out non d
    df = df[~(df[int_type] == 'Unknown')].copy()

    df.loc[df[int_type] == 'Activation', int_type] = 'activate|expression'
    df.loc[df[int_type] == 'Repression', int_type] = 'inhibit|repression'

    # Merge duplicate rows while maintaining interaction type and pmid
    int_joined = df.groupby(['source', 'target'])[int_type].apply(
        '|'.join).reset_index()
    pmid_joined = df.groupby(['source', 'target'])['pmid'].apply(
        '|'.join).reset_index()

    # merge to get one dataframe
    df = pd.merge(int_joined, pmid_joined, on=['source', 'target'])

    df['databaseSource'] = 'TRRUST'

    protein_graph = nx.from_pandas_edgelist(
        df,
        'source',
        'target',
        edge_attr=[int_type, 'pmid'],
        create_using=nx.DiGraph()
    )

    nodes = set(df[['source', 'target']].values.flatten())

    # add names to graph
    for n in nodes:
        protein_graph.add_node(n, databaseSource='TRRUST', speciesType='gene')
    nx.write_gpickle(protein_graph, _p_name)


def load_trrust(fresh_download=False, verbose=False):
    """
    Load reactome functional interaction network

    Parameters
    ----------
    fresh_download: bool
        Download fresh network
    verbose : bool

    Returns
    -------
    nx.DiGraph
    """
    if not os.path.exists(_p_name) or fresh_download:
        download_trrust()
        if not os.path.exists(_p_name):
            raise AssertionError("Error downloading TRRUST. ")
    tmp_graph = nx.read_gpickle(_p_name)
    if verbose:
        log.info("TRRUST network : {} nodes and {} edges".format(
            len(tmp_graph.nodes), len(tmp_graph.edges)))
    return tmp_graph


if __name__ == '__main__':
    load_trrust(fresh_download=True, verbose=True)



"""
Manually constructed mapping to most common description.

"""

edge_standards = {
    'activation'                        : 'activate',
    'activator'                         : 'activate',
    'potentiator'                       : 'activate',
    'oxidation'                         : 'oxidate',
    'palmitoylation'                    : 'palmitoylate',
    'inducer'                           : 'expression',
    'stimulator'                        : 'expression',
    'suppressor'                        : 'repression',
    'transcriptional activation'        : 'expression',
    'transcriptional inhibition'        : 'repression',
    'blocker'                           : 'inhibit',
    'inhibitor'                         : 'inhibit',
    'inhibition'                        : 'inhibit',
    'inhibitor, competitive'            : 'inhibit',

    'proteolytic processing'            : 'cleavage',
    'cleavage'                          : 'cleavage',
    'stabilization'                     : 'stabilization',
    'translation regulation'            : 'translation',

    'hydroxylation'                     : 'hydroxylation',
    'lipidation'                        : 'lipidation',
    'guanine nucleotide exchange factor': 'gtpase-activate',
    'gtpase-activating protein'         : 'gtpase-activate',

    # binding
    'binding'                           : 'binding',
    'binding/association'               : 'binding',
    'binder'                            : 'binding',
    'complex'                           : 'binding',
    'dissociation'                      : 'binding',

    # indirect/missing
    'indirect effect'                   : 'indirect',
    'missing interaction'               : 'indirect',

    'state change'                      : 'stateChange',
    'relocalization'                    : 'relocalization',

    's-nitrosylation'                   : 's-nitrosylation',
    'acetylation'                       : 'acetylate',
    'tyrosination'                      : 'tyrosinate',
    'ubiquitination'                    : 'ubiquitinate',
    'methylation'                       : 'methylate',
    'glycosylation'                     : 'glycosylate',
    'sumoylation'                       : 'sumoylate',
    'ribosylation'                      : 'ribosylate',
    'neddylation'                       : 'neddylate',
    'desumoylation'                     : 'desumoylate',
    'deneddylation'                     : 'deneddylate',
    'demethylation'                     : 'demethylate',
    'deacetylation'                     : 'deacetylate',
    'trimethylation'                    : 'trimethylate',
    'desensitize the target'            : 'inhibit',
    'deubiquitination'                  : 'deubiquitinate',
    'nedd(rub1)ylation'                 : 'neddy(rub1)late',

    'dephosphorylation': 'dephosphorylate',
    'phosphorylation': 'phosphorylate',

    'negative modulator': 'inhibit',
    'inhibitory allosteric modulator': 'allosteric|inhibit',
    'allosteric modulator': 'allosteric|modulate',
    'positive allosteric modulator': 'activate|allosteric',
    'positive modulator': 'activate',


    # chemical related
    'compound': 'chemical',
    'product of': 'chemical',
    'ligand': 'chemical',
    'cofactor': 'chemical',
    'multitarget': 'chemical',
    'small molecule catalysis': 'activate|catalyst|chemical',
    'chemical activation': 'activate|chemical',
    'chemical inhibition': 'chemical|inhibit',
    'partial agonist': 'activate|chemical',
    'inverse agonist': 'activate|chemical',
    'agonist': 'activate|chemical',
    'antagonist': 'inhibit|chemical',
    'partial antagonist': 'inhibit|chemical',
}



import os
import operator
import networkx as nx
from collections import OrderedDict
from magine.networks.standards import edge_standards
from magine.mappings.gene_mapper import GeneMapper


def create_dict_from_node_attributes(net, attribute):
    """

    Parameters
    ----------
    net : nx.DiGraph
    attribute : str

    Returns
    -------
    OrderedDict

    """
    attr_to_node = {}
    for i, j in net.nodes(attribute):
        if j not in attr_to_node:
            attr_to_node[j] = set()
        attr_to_node[j].add(i)
    return OrderedDict(
        sorted(attr_to_node.items(), key=operator.itemgetter(0)))


def delete_disconnected_network(full_graph):
    """
    Delete disconnected parts of a provided network

    Parameters
    ----------
    full_graph : networkx.DiGraph

    Examples
    --------
    >>> from magine.networks.utils import delete_disconnected_network
    >>> from networkx import DiGraph
    >>> g = DiGraph()
    >>> g.add_nodes_from(['a', 'b', 'c'])
    >>> g.add_edge('a', 'b')
    >>> large_g_connected = delete_disconnected_network(g)
    >>> sorted(large_g_connected.nodes())
    ['a', 'b']
    >>> large_g_connected.edges()
    [('a', 'b')]
    """
    copy_network = full_graph.copy()
    tmp_g = copy_network.to_undirected()
    sorted_graphs = sorted(nx.connected_component_subgraphs(tmp_g), key=len,
                           reverse=True)
    for i in range(1, len(sorted_graphs)):
        copy_network.remove_nodes_from(sorted_graphs[i].nodes())
    return copy_network


def add_attribute_to_network(graph, list_to_add_attribute, attribute,
                             true_term, false_term='false'):
    """

    Parameters
    ----------
    graph : networkx graph
    list_to_add_attribute : list
        list of nodes in graph to add attribute to
    attribute : str
        attribute to add to graph
    true_term : str
        value to add for the attribute provided True
    false_term : str
        value to add if attribute is false

    Returns
    -------
    nx.DiGraph

    Examples
    --------
    >>> from magine.networks.utils import add_attribute_to_network
    >>> from networkx import DiGraph
    >>> g = DiGraph()
    >>> g.add_nodes_from(['a', 'b', 'c'])
    >>> new_g = add_attribute_to_network(g, ['a','b'], 'inTest', 'true')
    >>> new_g.node['a']
    {'inTest': 'true'}
    >>> new_g.node['c']
    {'inTest': 'false'}

    """
    tmp_g = graph.copy()
    nodes = set(tmp_g.nodes)
    set_of_positive = set(list_to_add_attribute)
    for i in nodes:
        if i in set_of_positive:
            tmp_g.node[i][attribute] = true_term
        else:
            tmp_g.node[i][attribute] = false_term

    return tmp_g


def add_color_graphviz_fmt(graph, list_to_paint, color):
    """

    Parameters
    ----------
    graph: networkx.DiGraph
    list_to_paint : list_like
    color : str


    Examples
    --------
    >>> from magine.networks.utils import add_color_graphviz_fmt
    >>> from networkx import DiGraph
    >>> g = DiGraph()
    >>> g.add_nodes_from(['a', 'b', 'c'])
    >>> new_g = add_color_graphviz_fmt(g, ['a','b'], 'red')
    >>> new_g = add_color_graphviz_fmt(new_g, ['c'], 'blue')
    >>> new_g.node['a']
    {'style': 'filled', 'color': 'black', 'measured': True, 'fillcolor': 'red'}
    >>> new_g.node['c']
    {'style': 'filled', 'color': 'black', 'measured': True, 'fillcolor': 'blue'}


    Returns
    -------
    tmp_g : networkx.DiGraph

    """
    tmp_g = graph.copy()
    nodes1 = set(tmp_g.nodes)
    for i in list_to_paint:
        if i in nodes1:
            tmp_g.node[i]['measured'] = True
            tmp_g.node[i]['color'] = 'black'
            tmp_g.node[i]['fillcolor'] = color
            tmp_g.node[i]['style'] = 'filled'

    return tmp_g


def compose(g, g_1):
    """Return a new graph of G composed with H.

    Composition is the simple union of the node sets and edge sets.
    The node sets of G and H do not need to be disjoint.

    Parameters
    ----------
    g, : nx.DiGraph
    g_1 : nx.DiGraph
       A NetworkX graph

    Examples
    --------
    >>> from magine.networks.utils import compose
    >>> from networkx import DiGraph
    >>> g = DiGraph()
    >>> g.add_edge('A', 'B')
    >>> h = DiGraph()
    >>> g.add_edge('A', 'C')
    >>> new_g = compose(g, h)
    >>> print(sorted(new_g.nodes))
    ['A', 'B', 'C']
    >>> print(sorted(new_g.edges))
    [('A', 'B'), ('A', 'C')]

    Returns
    -------
    C: A new graph  with the same type as G

    """

    new_g = nx.DiGraph()

    _add_nodes(g, new_g)
    _add_nodes(g_1, new_g)

    _add_edges(g, new_g)
    _add_edges(g_1, new_g)

    return new_g


def compose_all(graphs):
    """Return the composition of all graphs.

    Composition is the simple union of the node sets and edge sets.
    The node sets of the supplied graphs need not be disjoint.

    Parameters
    ----------
    graphs : list
       List of NetworkX graphs

    Returns
    -------
    new_g : A graph with the same type as the first graph in list

    """
    for n, g in enumerate(graphs):
        if n != 0:
            new_g = compose(new_g, g)
        else:
            new_g = g
    return new_g


def remove_isolated_nodes(net):
    """
    Removes nodes with no edges from network.

    It modifies the provided graph inplace.

    Parameters
    ----------
    net : nx.DiGraph

    """
    to_remove = set()
    for i in net.nodes:
        if len(list(net.predecessors(i))) == 0 and \
                len(list(net.successors(i))) == 0:
            to_remove.add(i)
    net.remove_nodes_from(to_remove)


def _add_nodes(old_network, new_network):
    new_nodes = set(new_network.nodes)
    for i, data in old_network.nodes(data=True):
        if i not in new_nodes:
            new_network.add_node(i, **data)
        else:
            existing_info = new_network.node[i]
            for n, d in data.items():
                if n not in existing_info:
                    new_network.node[i][n] = d
                else:
                    additions = set(d.split('|'))
                    if isinstance(existing_info[n], list):
                        old = set(existing_info[n][0].split('|'))
                    else:
                        old = set(existing_info[n].split('|'))
                    additions.update(old)
                    new_network.node[i][n] = '|'.join(sorted(additions))


def _add_edges(current_network, new_network):
    edges = set(new_network.edges)
    for i, j, data in current_network.edges(data=True):
        if (i, j) not in edges:
            new_network.add_edge(i, j, **data)
        else:
            existing_info = new_network.get_edge_data(i, j)
            for n, d in data.items():
                if n not in existing_info:
                    new_network[i][j][n] = d
                else:
                    additions = set(d.split('|'))
                    additions.update(set(existing_info[n].split('|')))
                    new_network[i][j][n] = '|'.join(sorted(additions))


def standardize_edge_types(network):
    to_remove = set()
    for source, target, data in network.edges(data=True):
        if 'interactionType' in data:
            edge_type = data['interactionType']
            edge_type = set(i for i in edge_type.split('|'))

            for k, v in edge_standards.items():
                if k in edge_type:
                    edge_type.remove(k)
                    edge_type.add(v)
            for i in ['', ' ']:
                if i in edge_type:
                    edge_type.remove(i)

            edge_type = '|'.join(sorted(edge_type))

            if edge_type in ('', 'reaction', 'indirect'):
                to_remove.add((source, target))
            else:
                network[source][target]['interactionType'] = edge_type
    for source, target in to_remove:
        network.remove_edge(source, target)


def append_attribute_to_network(graph, list_to_add_attribute, attribute,
                                true_term, delimiter=''):
    """

    Parameters
    ----------
    graph : networkx graph
    list_to_add_attribute : list
        list of nodes in graph to add attribute to
    attribute : str
        attribute to add to graph
    true_term : str
        value to add for the attribute provided True
    delimiter : str


    Returns
    -------
    out : networkx graph


    >>> from networkx import DiGraph
    >>> from magine.networks.utils import append_attribute_to_network
    >>> g = DiGraph()
    >>> g.add_nodes_from(['a'])
    >>> new_g = append_attribute_to_network(g, ['a'], 'attribute', 'one')
    >>> new_g.node['a']
    {'attribute': 'one'}
    >>> new_g = append_attribute_to_network(new_g, ['a'], 'attribute', 'two', delimiter=',')
    >>> new_g.node['a']
    {'attribute': 'one,two'}

    """
    tmp_g = graph.copy()
    nodes = set(tmp_g.nodes())
    set_of_positive = set(list_to_add_attribute)
    for i in nodes:
        if i in set_of_positive:
            if attribute in tmp_g.node[i].keys():
                new_attr = tmp_g.node[i][attribute] + delimiter + true_term
                tmp_g.node[i][attribute] = new_attr
            else:
                tmp_g.node[i][attribute] = true_term

    return tmp_g


def trim_sink_source_nodes(network, list_of_nodes=None,
                           remove_self_edge=False):
    """
    Trim graph by removing nodes that are not in provided list if source/sink


    Parameters
    ----------
    network : networkx.DiGraph
    list_of_nodes : list_like
        list of species that are important if sink/source
    remove_self_edge : bool
        Remove self edges
    Returns
    -------

    """
    tmp_network = network.copy()
    if remove_self_edge:
        tmp_network = remove_self_edges(tmp_network)
    tmp1 = _trim(tmp_network, list_of_nodes)
    tmp2 = _trim(tmp_network, list_of_nodes)
    while tmp1 != tmp2:
        tmp2 = tmp1
        tmp1 = _trim(tmp_network, list_of_nodes)
    return tmp_network


def remove_self_edges(network):
    tmp_network = network.copy()
    tmp_network.remove_edges_from(tmp_network.selfloop_edges())
    return tmp_network


def _trim(network, list_of_nodes=None):
    if list_of_nodes is not None:
        list_of_nodes = set(list_of_nodes)
    found, not_found = 0., 0.
    nodes = set(network.nodes())
    in_dict = network.in_degree(nbunch=nodes)
    out_dict = network.out_degree(nbunch=nodes)
    for i in nodes:
        if list_of_nodes is not None:
            if i in list_of_nodes:
                found += 1
                continue
        if in_dict[i] == 1 and out_dict[i] == 0:
            network.remove_node(i)
        elif in_dict[i] == 0 and out_dict[i] == 1:
            network.remove_node(i)
        else:
            not_found += 1

    len_nodes = len(network.nodes())
    # print("{} found, {} not found".format(found, not_found))
    # print("{}% of {} nodes".format(found / len_nodes * 100, len_nodes))
    return len_nodes


def subtract_network_from_network(net1, net2):
    """
    subtract one network from another

    Parameters
    ----------
    net1 : networkx.DiGraph
        Base network, from which to subtract nodes from second network
    net2 : networkx.DiGraph

    Returns
    -------
    networkx.DiGraph

    """
    copy_graph1 = net1.copy()
    nodes1 = set(net1.nodes)
    nodes2 = set(net2.nodes)
    # we want to remove all nodes2 from the network.
    # Using intersection between two nodes prevents failure if a node
    # in nodes2 is not in the current network.
    copy_graph1.remove_nodes_from(nodes2.intersection(nodes1))

    return copy_graph1


def add_data_to_graph(network, exp_data):
    """ Add standard attributes to graph from data

    Parameters
    ----------
    network : nx.DiGraph
    exp_data : magine.data.experimental_data.ExperimentalData

    Returns
    -------

    """
    n_copy = network.copy()
    measured = set(exp_data.species.id_list)
    sig_measured = set(exp_data.species.sig.id_list)
    # seed species
    n_copy = add_attribute_to_network(n_copy, sig_measured,
                                      'sigMeasured', 'red', 'blue')

    # background
    n_copy = add_attribute_to_network(n_copy, measured,
                                      'measured', 'red', 'blue')
    # This retrieves a dictionary of where the keys are from the 'source'
    # of the data and values are lists of species
    m, sig_m = exp_data.get_measured_by_datatype()

    # add attribute if node is measured per 'source' of data
    for exp_type, spec in m.items():
        # this just cleans up non alpha-numeric characters
        attr_name = exp_type.replace('_', '')
        attr_name = attr_name.replace('-', '')
        network = add_attribute_to_network(network, spec, attr_name,
                                           'red', 'blue')
    # add labels for if node is measured in any of our samples
    for time, spec in zip(exp_data.species.sig.sample_ids,
                          exp_data.species.sig.by_sample):
        time = 'sample{}'.format(time)
        network = add_attribute_to_network(network, spec, time, 'red', 'blue')
    return n_copy



import logging
import os

import networkx as nx
import pandas as pd

from magine.data.storage import network_data_dir
from magine.networks.standards import edge_standards

_p_name = os.path.join(network_data_dir, 'signor.p.gz')
from magine.logging import get_logger

logger = get_logger(__name__, log_level=logging.INFO)


def download_signor():
    logger.info("Downloading SIGNOR")
    col_names = [
        'ENTITYA', 'TYPEA', 'IDA', 'DATABASEA', 'ENTITYB', 'TYPEB', 'IDB',
        'DATABASEB', 'EFFECT', 'MECHANISM', 'RESIDUE', 'SEQUENCE', 'TAX_ID',
        'CELL_DATA', 'TISSUE_DATA', 'MODULATOR_COMPLEX', 'TARGET_COMPLEX',
        'MODIFICATIONA', 'MODASEQ', 'MODIFICATIONB', 'MODBSEQ', 'PMID',
        'DIRECT', 'SENTENCE', 'SIGNOR_ID', 'NA1', 'NA2', 'NA3']

    table = pd.read_csv('https://signor.uniroma2.it/getData.php?organism=9606',
                        names=col_names, delimiter='\t', index_col=None,
                        error_bad_lines=False, encoding='utf-8'
                        )
    # filter out non direct
    table = table.loc[table['DIRECT'] == 't']

    # Filter out non descriptive
    table = table.loc[~table['MECHANISM'].isnull()]

    # Drop SIGNOR edges, these are generally complexes
    table = table[~(table['DATABASEA'] == 'SIGNOR')]
    table = table[~(table['DATABASEB'] == 'SIGNOR')]

    # Not sure what they mean, so will remove. Ideally other DBs have this info
    table = table[~(table['MECHANISM'] == 'post transcriptional regulation')]

    col_a = ['ENTITYA', 'TYPEA', 'IDA', 'DATABASEA']
    col_b = ['ENTITYB', 'TYPEB', 'IDB', 'DATABASEB']
    cols = ['name', 'species_type', 'id', 'db']
    species_a = table[col_a].copy()
    species_b = table[col_b].copy()
    species_a.rename(columns={i: j for i, j in zip(col_a, cols)}, inplace=True)
    species_b.rename(columns={i: j for i, j in zip(col_b, cols)}, inplace=True)
    species_a.drop_duplicates(inplace=True)
    species_b.drop_duplicates(inplace=True)
    all_species = pd.concat([species_a, species_b])
    all_species.drop_duplicates(inplace=True)

    def map_to_activate_inhibit(row):
        effect = ''
        mechanism = row['MECHANISM']
        if 'down-regulates' in row['EFFECT']:
            effect = 'inhibit'
        elif 'up-regulates' in row['EFFECT']:
            effect = 'activate'
        if mechanism in edge_standards:
            mechanism = edge_standards[mechanism]
        elif mechanism == 'transcriptional regulation':
            if effect == 'inhibit':
                mechanism = 'repression'
            elif effect == 'activate':
                mechanism = 'expression'
        if effect == '':
            return mechanism
        else:
            return "|".join([effect, mechanism])

    # relabel edge types
    table['interactionType'] = table.apply(map_to_activate_inhibit, axis=1)
    table['databaseSource'] = 'SIGNOR'
    table['pmid'] = table['PMID']

    table['source'] = table['ENTITYA']
    table['target'] = table['ENTITYB']

    protein_graph = nx.from_pandas_edgelist(
        table,
        'source',
        'target',
        edge_attr=['interactionType', 'databaseSource'],
        create_using=nx.DiGraph()
    )
    # add names to graph
    for row in all_species.values:
        name, species_type, id_name, db = row
        if species_type != 'protein':
            species_type = 'compound'
        if species_type == 'protein':
            species_type = 'gene'

        protein_graph.add_node(name, databaseSource='SIGNOR',
                               speciesType=species_type)

    nx.write_gpickle(protein_graph, _p_name)
    logger.info("Done downloading SIGNOR")


def load_signor(fresh_download=False):
    """
    Load reactome functional interaction network

    Parameters
    ----------
    fresh_download: bool
        Download fresh network
    verbose : bool

    Returns
    -------
    nx.DiGraph
    """
    if not os.path.exists(_p_name) or fresh_download:
        print("Downloading Signor network!")
        download_signor()
        if not os.path.exists(_p_name):
            raise FileNotFoundError("Error downloading reactome FI. ")
    tmp_graph = nx.read_gpickle(_p_name)

    logger.info("SIGNOR : {} nodes and {} edges".format(len(tmp_graph.nodes),
                                                        len(tmp_graph.edges)))
    return tmp_graph


if __name__ == '__main__':
    download_signor()



import networkx as nx
from nose.tools import ok_

import magine.networks.databases.kegg_kgml as kegg_tools
from magine.mappings.maps import convert_all

digraph = nx.classes.digraph.DiGraph


def test_construction():
    network = kegg_tools.pathway_id_to_network('hsa00040')
    ok_(isinstance(network, digraph))
    network = kegg_tools.pathway_id_to_network('hsa04978')
    ok_(isinstance(network, digraph))
    network = kegg_tools.pathway_id_to_network('hsa04071')
    ok_(isinstance(network, digraph))
    network = kegg_tools.pathway_id_to_network('hsa04973')
    ok_(isinstance(network, digraph))


def test_convert_all():
    network = kegg_tools.pathway_id_to_network('hsa00040')
    convert_all(network, verbose=True)



import json
import uuid

import networkx as nx
import numpy as np
from IPython.display import display, HTML

from magine.html_templates.html_tools import env
from magine.networks.exporters import nx_to_json
from .cyjs_options import layouts, styles


def draw_cyjs(graph, add_parent=False, layout='cose-bilkent',
              bg_color='white', default_node_color='white', **layout_args):
    """ Renders a graph using cytoscape.js

    Parameters
    ----------
    graph : nx.DiGraph
    add_parent : bool
        Group together nodes that share 'termName'.
    layout : str
        Layout
    bg_color
    default_node_color : str
        Will only be used if color is not already set for node
    layout_args : dict
        Any additional arguments for use in cytoscape.js. Examples can be found
        in cyjs_options.py

    Returns
    -------

    """
    # Dont think users need these
    height = 700
    width = 100
    g_copy = graph.copy()
    if add_parent:
        g_copy = _add_parent_term(g_copy)

    _scale_edges(g_copy)
    _set_node_color(g_copy, default_node_color)

    d = nx_to_json(g_copy)
    d['background'] = bg_color
    d['uuid'] = "cy" + str(uuid.uuid4())
    d['widget_width'] = str(width)
    d['widget_height'] = str(height)

    if layout not in layouts:
        raise Exception(
            "layout {} is not in {}".format(layout, layouts.keys())
        )
    else:
        layout_opts = layouts[layout].copy()
        layout_opts.update(layout_args)

    d['layout_json'] = json.dumps(layout_opts)
    d['style_json'] = json.dumps(styles['default'])
    d['fitbutton'] = "fit{}".format(uuid.uuid4())

    template = env.get_template('subgraph_2.html')
    display(HTML(template.render(d)))
 

def render_graph(graph, add_parent=False, default_color='white'):
    g_copy = graph.copy()

    _scale_edges(g_copy)

    if add_parent:
        g_copy = _add_parent_term(g_copy)
    _set_node_color(g_copy, default_color)
    d = nx_to_json(g_copy)
    u_name = "cy{}".format(uuid.uuid4())
    d['uuid'] = u_name
    d['style_json'] = json.dumps(styles['default'])

    edge_types = set()
    for i, j, data in graph.edges(data=True):
        if 'interactionType' in data:
            for e in data['interactionType'].split('|'):
                edge_types.add(e)

    d['edge_list'] = list(edge_types)

    fname_temp = '{}.html'.format(u_name)
    subgraph_html = env.get_template('subgraph.html')
    template = env.get_template('main_view.html')

    with open(fname_temp, 'w') as f:
        f.write(subgraph_html.render(d))

    display(HTML(template.render(name=u_name, filename=fname_temp)))


def _set_node_color(net, color):
    for i in net.nodes:
        if 'color' not in net.node[i]:
            net.node[i]['color'] = color


def _scale_edges(net):
    # store in a weird format. An EdgeView should maintain same ordering when
    # enumerating, but just in case going to keep index pairs for placing width
    vals = [[[i, j], val] for i, j, val in net.edges.data('weight', default=5)]
    pairs = [pair for pair, _ in vals]
    widths = np.array([val for _, val in vals])

    # normalize between 1, 100
    edge_width = np.interp(widths, (widths.min(), widths.max()), (3, 10))
    for width, (i, j) in zip(edge_width, pairs):
        net.edges[i, j]['width'] = width


def _add_parent_term(graph):
    g_copy = graph.copy()
    new_nodes = set()
    for i, data in graph.nodes(data=True):
        if 'termName' in data:
            term = data['termName']
            g_copy.node[i]['parent'] = term
            new_nodes.add(term)
    for each in new_nodes:
        g_copy.add_node(each)
    return g_copy


if __name__ == '__main__':
    g = nx.DiGraph()
    g.add_edge('a', 'b', weight=10)
    g.add_edge('b', 'c', weight=2)
    g.add_edge('b', 'a')
    g.add_edge('b', 'p', weight=100)
    _scale_edges(g)

    # render_graph(g)



import json

import igraph
import networkx as nx
import pydot
from nose.tools import ok_

import magine.networks.exporters as exporters


def test_nx_to_igraph():
    g = nx.DiGraph()
    g.add_edge('a', 'b')
    i_g = exporters.nx_to_igraph(g)
    ok_(isinstance(i_g, igraph.Graph))


def ordered(obj):
    if isinstance(obj, dict):
        return sorted((k, ordered(v)) for k, v in obj.items())
    if isinstance(obj, list):
        return sorted(ordered(x) for x in obj)
    else:
        return obj


def test_nx_to_json():
    g = nx.DiGraph()
    g.add_edge('a', 'b')
    json_g = exporters.nx_to_json(g)
    answer = {'edges': '[{"data": {"source": "a", "target": "b"}}]',
              'nodes': '[{"data": {"id": "a", "name": "a"}},'
                       ' {"data": {"id": "b", "name": "b"}}]'}
    a_edges = json.loads(json_g['edges'])
    b_edges = json.loads(answer['edges'])
    a_nodes = json.loads(json_g['nodes'])
    b_nodes = json.loads(answer['nodes'])

    ok_(ordered(a_edges) == ordered(b_edges))
    ok_(ordered(a_nodes) == ordered(b_nodes))


def test_nx_to_dot():
    g = nx.DiGraph()
    g.add_edge('a', 'b')
    dot_g = exporters.nx_to_dot(g)
    ok_(isinstance(dot_g, pydot.Dot))



"""
Database downloads.

MAGINE downloads network information from
* Reactome Functional Interactions
* HMDB
* BioGrid
* KEGG
* Signor
With the exception of KEGG and HMDB, all databases are downloaded and processed
with pandas. KEGG is downloaded using Bioservices. HMDB is download in xml
format and processed using lxml parser.

"""
from .biogrid_interactions import download_biogrid, load_biogrid_network
from .hmdb import load_hmdb_network
from .kegg_kgml import download_kegg, load_kegg, load_kegg_mappings
from .reactome_functional_interaction import download_reactome_fi, \
    load_reactome_fi
from .signor import download_signor, load_signor

__all__ = ['load_reactome_fi', 'download_reactome_fi',
           'load_biogrid_network', 'download_biogrid',
           'load_signor', 'download_signor',
           'load_kegg_mappings', 'load_kegg',
           'load_hmdb_network']



import matplotlib.pyplot as plt
import networkx as nx

from magine.networks.visualization import draw_igraph, draw_mpl


def test_igraph():
    g = nx.DiGraph()
    g.add_node('A', termName='one')
    g.add_node('B', termName='one')
    g.add_node('C', termName='two')
    g.add_node('D', termName='two')
    g.add_edge('A', 'B')
    g.add_edge('C', 'B')
    g.add_edge('C', 'D')

    plot, pos = draw_igraph(g, )
    plot, pos = draw_igraph(g, cluster=True)
    plot, pos = draw_igraph(g, cluster=True, positions=pos)


def test_mpl():
    g = nx.DiGraph()
    g.add_node('A', termName='one')
    g.add_node('B', termName='one')
    g.add_node('C', termName='two')
    g.add_node('D', termName='two')
    g.add_edge('A', 'B')
    g.add_edge('C', 'B')
    g.add_edge('C', 'D')
    for i in ['circular_layout', 'random_layout', 'shell_layout',
              'spring_layout', 'spectral_layout',  # 'dot', 'neato', 'fdp',
              'fruchterman_reingold_layout']:
        fig = draw_mpl(g, i)
        plt.close()



import logging
import os
import networkx as nx
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import magine.networks.utils as nt
import magine.networks.databases as db
from magine.mappings.chemical_mapper import ChemicalMapper
from magine.mappings.gene_mapper import GeneMapper

try:
    import cPickle as pickle
except ImportError:
    import pickle
from magine.logging import get_logger

# logger = get_logger("magine.networks.network_generator", log_level=logging.INFO)
logger = get_logger(__name__, log_level=logging.INFO)


def build_network(seed_species, species='hsa', save_name=None,
                  all_measured_list=None, trim_source_sink=False,
                  use_reactome=True, use_hmdb=False,
                  use_biogrid=True, use_signor=True, verbose=False):
    """
    Construct a network from a list of gene names.

    Parameters
    ----------

    seed_species : list
        list of genes to construct network
    save_name : str, optional
        output name to save network. Will save one before and after ID
        conversion
    species : str
        species of proteins ('hsa': human, 'mmu':murine)
    all_measured_list : list
        list of all species that should be considered in network
    use_reactome : bool
        Add ReactomeFunctionalInteraction reaction to network
    use_biogrid : bool
        Add BioGrid reaction to network
    use_hmdb : bool
        Add HMDB reaction to network
        all_measured_list
    use_signor : bool
        Add SIGNOR reaction to network
    trim_source_sink : bool, optional
        Remove source and sink nodes if they are not measured in network
    verbose : bool

    Returns
    -------
    networkx.DiGraph
    """
    cm = ChemicalMapper()
    path_to_graph, node_to_path = db.load_kegg_mappings(species)

    seed_species = set(x.upper() for x in seed_species)
    updated_accession = set()
    old_accession = set()
    # This maps HMDB numbers to the new numbering format
    for i in seed_species:
        if i.startswith('HMDB'):
            if i in cm.hmdb_accession_to_main:
                old_accession.add(i)
                updated_accession.add(cm.hmdb_accession_to_main[i][0])

    seed_species.difference_update(old_accession)
    seed_species.update(updated_accession)

    seeds_in_kegg = seed_species.intersection(node_to_path)

    pathway_list = set()
    for seed in seeds_in_kegg:
        pathway_list.update(node_to_path[seed])

    graph_list = []
    for each in pathway_list:
        tmp = path_to_graph[each]
        if len(tmp.edges) == 0:
            continue
        graph_list.append(tmp)
    end_network = nt.compose_all(graph_list)

    if all_measured_list is None:
        all_measured_set = set(i.upper() for i in end_network.nodes)
    else:
        all_measured_set = set(str(x).upper() for x in all_measured_list)

    all_measured_set.update(seed_species)
    hmdb_ids = set(i for i in all_measured_set if i.startswith('HMDB'))
    updated_accession = set()
    old_accession = set()
    for i in hmdb_ids:
        if i in cm.hmdb_accession_to_main:
            all_measured_set.remove(i)
            all_measured_set.add(cm.hmdb_accession_to_main[i][0])
    networks_to_expand = []
    logger.info("Gathering networks")
    if use_hmdb:
        networks_to_expand.append(db.load_hmdb_network())

    if use_reactome:
        networks_to_expand.append(db.load_reactome_fi())

    if use_biogrid:
        networks_to_expand.append(db.load_biogrid_network())

    if use_signor:
        networks_to_expand.append(db.load_signor())
    logger.info("Merging networks")
    if len(networks_to_expand) != 0:
        entire_expansion_network = nt.compose_all(networks_to_expand)
        end_network = expand_by_db(end_network, entire_expansion_network,
                                   all_measured_set)

    logger.info("Trimming network")
    # makes all similar edge names the same
    nt.standardize_edge_types(end_network)
    # removes everything not connected to the largest graph
    end_network = nt.delete_disconnected_network(end_network)

    if trim_source_sink:
        end_network = nt.trim_sink_source_nodes(end_network, all_measured_list,
                                                remove_self_edge=True)
    if save_name is not None:
        nx.write_gml(end_network, '{}.gml'.format(save_name))
        nx.write_gpickle(end_network, '{}.p'.format(save_name))

    final_nodes = set(end_network.nodes)
    n_hits = len(seed_species.intersection(final_nodes))
    logger.info('Network has {} nodes and {} edges'.format(
        len(final_nodes), len(end_network.edges))
    )

    logger.info("Found {} of {} seed species in network".format(
        n_hits, len(seed_species))
    )
    if all_measured_list is not None:
        n_measured_hits = len(set(all_measured_list).intersection(final_nodes))
        logger.info("Found {} of {} background species in network"
                    "".format(n_measured_hits, len(all_measured_list)))

    return end_network


def expand_by_db(starting_network, expansion_source, measured_list,
                 verbose=False):
    """ add reference network to main network

    Parameters
    ----------
    starting_network : nx.DiGraph
    expansion_source : nx.DiGraph
    measured_list : list_like
    verbose : bool

    Returns
    -------
    new_graph : nx.DiGraph
    """

    new_graph = nx.DiGraph()
    measured_set = set(measured_list)
    current_nodes = set(starting_network.nodes)

    # gathers possible nodes to expand

    # must be in new nodes AND measured set
    nodes_to_check = set(expansion_source.nodes).intersection(measured_set)

    # add all current nodes, might be edges between them that are missed
    nodes_to_check.update(current_nodes)

    added_nodes = set()

    for i, j, k in expansion_source.edges(data=True):
        if i in nodes_to_check and j in nodes_to_check:
            added_nodes.add(i)
            added_nodes.add(j)
            new_graph.add_edge(i, j, **k)

    for node in added_nodes:
        new_graph.add_node(node, **expansion_source.node[node])

    new_graph = nt.compose(starting_network, new_graph)
    logger.info("\t\t\tbefore\tafter")
    logger.info("\tNodes\t{}\t{}".format(len(starting_network.nodes),
                                         len(new_graph.nodes)))

    logger.info("\tEdges\t{}\t{}".format(len(starting_network.edges),
                                         len(new_graph.edges)))

    return new_graph


def create_background_network(save_name='background_network',
                              fresh_download=False, verbose=True,
                              create_overlap=False):
    """

    Parameters
    ----------
    save_name : str
        Name of the network
    fresh_download : bool
        Download a fresh copy of the databases
    verbose: bool
        Print information about the databases
    create_overlap : bool
        Creates a figure comparing the databses
    Returns
    -------
    nx.DiGraph
    """
    logger.info("Generating background network from all databases")
    kegg_network = db.load_kegg(fresh_download=fresh_download)
    hmdb_network = db.load_hmdb_network(fresh_download=fresh_download)
    biogrid_network = db.load_biogrid_network(fresh_download=fresh_download)
    signor_network = db.load_signor(fresh_download=fresh_download)
    reactome_network = db.load_reactome_fi()
    network_list = [hmdb_network, kegg_network, biogrid_network,
                    reactome_network, signor_network]
    names = ['hmdb', 'kegg', 'biogrid', 'reactome', 'signor']

    def find_overlap(n1, n2):
        nodes1 = set(n1.nodes())
        nodes2 = set(n2.nodes())
        e1 = set(n1.edges())
        e2 = set(n2.edges())
        edge_overlap = len(e2.intersection(e1))
        node_overlap = len(nodes1.intersection(nodes2))
        return node_overlap, edge_overlap

    if create_overlap:
        db_maps = {i: j for i, j in zip(names, network_list)}
        n_dbs = len(names)

        pal = sns.light_palette("purple", as_cmap=True)
        node_mat = np.zeros((n_dbs, n_dbs), dtype=np.int)
        edge_mat = np.zeros((n_dbs, n_dbs), dtype=np.int)

        for i in range(n_dbs):
            row = db_maps[names[i]]
            for j in range(i + 1, n_dbs):
                col = db_maps[names[j]]
                n_overlap, e_overlap = find_overlap(row, col)
                node_mat[i, j] = n_overlap
                node_mat[j, i] = node_mat[i, j]
                edge_mat[i, j] = e_overlap
                edge_mat[j, i] = edge_mat[i, j]

        fig = plt.figure(figsize=(10, 4))
        ax = fig.add_subplot(121)
        plt.title("Number of node overlaps")
        sns.heatmap(node_mat, fmt='d', annot=True, linewidths=0.02, cmap=pal,
                    yticklabels=names, xticklabels=names)
        plt.yticks(rotation=0)
        ax = fig.add_subplot(122)
        plt.title("Number of edge overlaps")
        sns.heatmap(edge_mat, fmt='d', annot=True, linewidths=0.02, cmap=pal,
                    yticklabels=names, xticklabels=names)
        plt.tight_layout()
        plt.yticks(rotation=0)
        plt.subplots_adjust(wspace=.3)
        plt.savefig('compare_network_dbs.png', dpi=300, bbox_inches='tight')
        plt.close()
    logger.info("Combinding networks")
    full_network = nt.compose_all(network_list)
    logger.info("Finished combinding networks")
    nt.standardize_edge_types(full_network)
    full_network = nt.delete_disconnected_network(full_network)
    n_nodes = len(full_network.nodes)
    n_edges = len(full_network.edges)
    logger.info("Background network {} nodes and {} edges".format(n_nodes,
                                                                  n_edges))

    nx.write_gpickle(full_network, '{}.p.gz'.format(save_name))
    nx.write_gml(full_network, '{}.gml'.format(save_name))
    return full_network


if __name__ == '__main__':
    create_background_network(fresh_download=False, verbose=True,
                              create_overlap=True)



import os

from magine.data.experimental_data import load_data

data_dir = os.path.dirname(__file__)
exp_data = load_data(os.path.join(os.path.join(data_dir, 'Data'),
                                      'example_apoptosis.csv'))



#!python
import logging

from magine.data.storage import clear_cached_dbs, create_storage_structure
from magine.logging import get_logger
import magine.mappings.databases.download_libraries as dl
import magine.networks.databases as nd

logger = get_logger(__name__, log_level=logging.INFO)


def download_id_mapping():
    dl.download_hgnc()
    dl.download_ncbi()
    dl.download_uniprot()


def download_network_dbs():
    dl.download_hmdb()
    nd.download_reactome_fi()
    nd.download_signor()
    nd.download_biogrid()


def run():
    import time

    clear_cached_dbs()
    create_storage_structure()
    logger.info("Downloading all network and ID mapping databases")
    st = time.time()
    download_id_mapping()
    download_network_dbs()
    et = time.time()
    logger.info("Took {} seconds".format(et - st))


if __name__ == '__main__':
    run()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import find_packages, setup

import magine

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), 'r') as f:
    long_description = f.read()

setup(
    name='MAGINE',
    version=magine.__version__,
    description='Package to analyze biological data.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    url='https://github.com/LoLab-VU/Magine',
    author='James Pino',
    author_email='james.ch.pino@gmail.com',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Science/Research',
    ],

    keywords=[
        'biological networks', 'biological pathways', 'enrichment analysis',
        'network analysis', 'visualization',
        'multi-omics', 'rnaseq', 'proteomics', 'metabolomics'
    ],

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['docs']),

    install_requires=[
        'bioservices',
        'defusedxml',
        'ipywidgets',
        'jinja2',
        'jupyter',
        'matplotlib==3.1.0', 'matplotlib-venn',
        'networkx>=2.1,<2.4',
        'numpy',
        'pandas',
        'pathos',
        'plotly==2.7',
        'py2cytoscape',
        'pycairo',
        'pydot',
        'pydotplus',
        'requests',
        'scipy',
        'seaborn',
        'sortedcontainers',
        'statsmodels',
        'wordcloud',
        'xlrd',
    ],

    test_suite='nose.collector',
    tests_require=['nose', 'coverage'],

    extras_require={
        'test': ['coverage'],
    },
    include_package_data=True,
    scripts=['scripts/create_template_project',
             'scripts/download_databases.py'],
    entry_points={
        'console_scripts': [
            'download_magine_databases=download_databases:run'
        ]
    }
)



import gzip
import logging
import math
import os

from bioservices import KEGG
import defusedxml.cElementTree as et
import networkx as nx

from magine.data.storage import network_data_dir
import magine.networks.utils as utils

try:
    import cPickle as pickle
except:
    import pickle

kegg = KEGG()
kegg.TIMEOUT = 100

from magine.logging import get_logger

logger = get_logger(__name__, log_level=logging.INFO)


def pathway_id_to_network(pathway_id, species='hsa'):
    """
    Creates a network from a KEGG pathway id

    Parameters
    ----------
    pathway_id : str
        KEGG id
    species : str

    Returns
    -------
    nx.DiGraph

    """
    kegg.organism = species
    list_of_kegg_pathways = [i[5:] for i in kegg.pathwayIds]

    if pathway_id not in list_of_kegg_pathways:
        raise AssertionError("{} not a KEGG pathway id".format(pathway_id))
    pathway = kegg.get(pathway_id, "kgml")
    graph, pathway_name = kgml_to_nx(pathway, species=species)
    return graph


def kgml_to_nx(xml_file, species='hsa'):
    """ Converts a kgml to nx.DiGraph

    Parameters
    ----------
    xml_file : str
    species : str

    Returns
    -------

    """
    try:
        tree = et.fromstring(xml_file)

    except TypeError:
        raise TypeError("This file ({})is messed up!".format(xml_file))
    pathway_local = nx.DiGraph()
    connecting_maps = []
    name_label_dict = {}
    organism = tree.get('org')
    pathway_name = tree.get('title')

    def _add_node(node, type_species):
        pathway_local.add_node(node, speciesType=type_species,
                               databaseSource='KEGG')

    def _add_edge(source, target, type_int):
        pathway_local.add_edge(source, target,
                               databaseSource='KEGG',
                               interactionType=type_int,
                               )

    if organism != species:
        print(organism, species)
        raise NotImplementedError("Error with species not matching from KEGG")

    # get all nodes of pathway
    for entry in tree.iter('entry'):

        # get node id
        node_id = entry.get('id')
        node_type = entry.get('type')
        if node_type in ('gene', 'compound'):
            name = entry.get('name')
            names = name.split()
            name_label_dict[node_id] = names
            for i in names:
                _add_node(i, node_type)
        else:
            # add everything that is not a gene or compound to exclude
            # orthologs are from other species
            # groups is just a trick for KGML to layout species together
            # maps point to other dbs
            if node_type not in ('map', 'ortholog', 'brite', 'group'):
                print("Not a gene or compound!!!")
                print(node_type, node_id)
            connecting_maps.append(node_id)

    # Add all relations of pathway
    for rel in tree.iter('relation'):
        e1 = rel.get('entry1')
        e2 = rel.get('entry2')
        if e1 in connecting_maps or e2 in connecting_maps:
            continue
        try:
            int_type = set()
            for interaction in rel.getiterator('subtype'):
                int_type.add(interaction.get('name'))
            int_type = '|'.join(int_type)
        except TypeError:
            continue
        if 'indirect' in int_type:
            continue
        one, two = name_label_dict[e1], name_label_dict[e2]
        for i in one:
            for j in two:
                _add_edge(i, j, int_type)

    # Add all reactions of pathway
    for reaction in tree.iter('reaction'):

        id_local = reaction.get('id')
        if id_local in connecting_maps:
            print(id_local)
            continue

        reaction_type = "chemical|reaction|{}".format(reaction.get('type'))
        subs = [str(i.get('name')) for i in reaction.getiterator('substrate')]
        prods = [str(i.get('name')) for i in reaction.getiterator('product')]
        enzyme = name_label_dict[id_local]

        for i in enzyme:
            for sub in subs:
                _add_edge(sub, i, type_int=reaction_type)

            for prod in prods:
                _add_edge(i, prod, type_int=reaction_type)

    return pathway_local, pathway_name


def download_kegg(species='hsa'):
    """
    Downloads every KEGG pathway to provided directory

    """
    if species == 'hsa':
        from magine.mappings.maps import convert_all
    kegg.organism = species
    list_of_kegg_pathways = [i[5:] for i in kegg.pathwayIds]

    n_pathways = len(list_of_kegg_pathways)
    logger.info("Downloading {} KEGG pathways".format(n_pathways))

    # keys are KEGG pathways ids, values are kegg pathways
    kegg_dict = dict()
    v = set()
    milestone_markers = range(0, 101, 10)

    for n, pathway_id in enumerate(list_of_kegg_pathways):
        pathway = kegg.get(pathway_id, "kgml")

        if pathway == 404:
            logger.info("%s ended with 404 error" % pathway_id)
            continue

        graph, pathway_name = kgml_to_nx(pathway, species=species)
        if species == 'hsa':
            graph = convert_all(graph, species)
        kegg_dict[pathway_id] = graph
        logger.debug('{} has {} nodes and {} edges'.format(
            pathway_name,
            len(graph.nodes),
            len(graph.edges))
        )
        logger.info("On pathway {} of {}".format(n, n_pathways))
        percent_done = int(math.floor(100 * n / n_pathways))
        if percent_done in milestone_markers:
            if percent_done not in v:
                logger.info("{}%".format(percent_done))
                v.add(percent_done)

    # create a dictionary mapping species to pathways
    node_to_path = dict()
    for i in kegg_dict:
        for node in kegg_dict[i].nodes:
            if node not in node_to_path:
                node_to_path[node] = set()
            node_to_path[node].add(i)

    # save kegg pathway its to networks
    n = '{}_kegg_path_ids_to_networks.p.gz'.format(species)
    save_path_id_to_graph = os.path.join(network_data_dir, n)
    save_gzip_pickle(save_path_id_to_graph, kegg_dict)

    # save nodes to pathways
    n = '{}_kegg_node_to_pathway.p.gz'.format(species)
    save_node_to_path = os.path.join(network_data_dir, n)
    save_gzip_pickle(save_node_to_path, node_to_path)
    logger.info("Finished downloading all KEGG pathways")
    return kegg_dict


def load_kegg(species='hsa', fresh_download=False):
    """
    Loads all KEGG pathways as a single network

    Parameters
    ----------
    species : species
        Default 'hsa'
    fresh_download : bool
        Download kegg new

    Returns
    -------
    nx.DiGraph
    """
    logger.info("Loading KEGG network")
    p_name = os.path.join(network_data_dir,
                          '{}_all_of_kegg.p.gz'.format(species))
    if os.path.exists(p_name) and not fresh_download:
        # load in network if it exists
        all_of_kegg = nx.read_gpickle(p_name)
    else:
        kegg_pathways = download_kegg(species=species)
        all_of_kegg = utils.compose_all(kegg_pathways.values())
        # relabel nodes to gene names
        if species == 'hsa':
            from magine.mappings.maps import convert_all
            all_of_kegg = convert_all(all_of_kegg, species=species)
        # save network
        nx.write_gpickle(all_of_kegg, p_name)
    logger.info("KEGG network {} nodes and {} edges".format(
        len(all_of_kegg.nodes), len(all_of_kegg.edges))
    )
    return all_of_kegg


def load_kegg_mappings(species, fresh_download=False):
    """
    Load mappings of kegg_pathway_id to nodes and nodes to kegg_pathway_id

    Parameters
    ----------
    species : str
        Species type, currently 'hsa' is the only species with automatic name
        conversion
    fresh_download : bool
        Download KEGG fresh

    Returns
    -------
    dict, dict
    """
    logger.info("Loading KEGG mapping")

    _kegg_id_to_pathway = '{}_kegg_path_ids_to_networks.p.gz'.format(species)
    save_path_id_to_graph = os.path.join(network_data_dir, _kegg_id_to_pathway)

    if not os.path.exists(save_path_id_to_graph) or fresh_download:
        download_kegg(species=species)

    _kegg_node_to_pathway = '{}_kegg_node_to_pathway.p.gz'.format(species)
    save_node_to_path = os.path.join(network_data_dir, _kegg_node_to_pathway)

    # load all networks
    return load_gz_p(save_path_id_to_graph), load_gz_p(save_node_to_path)


def save_gzip_pickle(file_name, obj):
    with gzip.open(file_name, 'wb') as f:
        f.write(pickle.dumps(obj, protocol=-1))


def load_gz_p(file_name):
    with gzip.open(file_name, 'rb') as f:
        data = f.read()
    try:
        return pickle.loads(data, encoding='utf-8')
    except:
        return pickle.loads(data)


if __name__ == '__main__':
    # download_kegg('hsa')
    g = load_kegg(fresh_download=True)
    # load_kegg('acb')



import os

import networkx as nx

from magine.networks.exporters import nx_to_dot, export_to_dot
from magine.networks.utils import add_color_graphviz_fmt

IPYTHON = False
try:
    from IPython.display import Image, display

    IPYTHON = True
except RuntimeError:
    pass


def draw_graphviz(network, layout='dot', image_format='png', width=800,
                  save_name=None):
    """ Draws an


    Parameters
    ----------
    network : nx.DiGraph
    layout : str
        Which graphviz engine to use to layout graph
    image_format: str
        image format (png, svg, pdf)
    width : int
        Width of image
    save_name : str, optional
        If you want to export to file

    Returns
    -------

    """
    net_copy = network.copy()
    net_copy.graph.update({'K': '1'})
    net_copy.graph.update({'repulsiveforce ': '1'})
    net_copy.graph.update({'overlap ': 'false'})
    net_copy.graph.update({'splines ': 'true'})
    if save_name is not None:
        if save_name.endswith(image_format):
            out_name = save_name
        else:
            out_name = '{}.{}'.format(save_name, image_format)
        nx_to_dot(net_copy).write(out_name, format=image_format,
                                  prog=layout)
        if IPYTHON and run_from_ipython():
            display(Image(out_name, width=width))
    else:
        img = nx_to_dot(net_copy).create(format='png', prog=layout)
        if IPYTHON and run_from_ipython():
            display(Image(img, width=width))


def paint_network_overtime(graph, list_of_lists, color_list, save_name,
                           labels=None, create_gif=False):
    """
    Adds color attribute to network over time.

    Parameters
    ----------
    graph : nx.DiGraph
        Network
    list_of_lists : list_like
        List of lists, where the inner list contains the node to add the
        color
    color_list : list_like
        list of colors for each time point
    save_name : str
        prefix for images to be saved
    labels: list_like
        list of labels to add to graph per sample

    """

    if len(list_of_lists) != len(color_list):
        print('Length of list of data must equal len of color list')
        return
    if labels is not None:
        if len(labels) != len(list_of_lists):
            print('Length of labels must be equal to len of data')
            return

    string = 'convert -delay 100 '
    tmp_graph = graph.copy()

    for n, i in enumerate(list_of_lists):
        graph2 = add_color_graphviz_fmt(tmp_graph, i, color_list[n])

        if labels is not None:
            graph2.graph['label'] = labels[n]
            graph2.graph['fontsize'] = 13

        s_name = '%s_%04i.png' % (save_name, n)
        draw_graphviz(graph2, layout='dot', save_name=s_name,
                      image_format='png')
        string += s_name + ' '
    string1 = string + '  %s.gif' % save_name
    string2 = string + '  %s.pdf' % save_name
    if create_gif:
        os.system(string1)
        os.system(string2)


def paint_network_overtime_up_down(graph, list_up, list_down, save_name,
                                   color_up='red', color_down='blue',
                                   labels=None, create_gif=False):
    """
    Adds color attribute to network over time and creates figure.

    Parameters
    ----------
    graph : nx.DiGraph
        Network
    list_up : list_like
        List of lists, where the inner list contains the node to add the
        color
    list_down : list_like
        list of colors for each time point
    color_up : str
        color for first list of species
    color_down : str
        color of second list of species
    save_name : str
        prefix for images to be saved
    labels: list_like
        list of labels to add to graph per sample
    create_gif : bool
        Create a gif from series of images


    """

    if len(list_up) != len(list_down):
        print('Length of list of data must equal len of color list')
        return
    if labels is not None:
        if len(labels) != len(list_down):
            print('Length of labels must be equal to len of data')
            return
    string = 'convert -delay 100 '
    tmp_graph = graph.copy()

    for n, (up, down) in enumerate(zip(list_up, list_down)):
        tmp_graph = add_color_graphviz_fmt(tmp_graph, up, color_up)
        tmp_graph = add_color_graphviz_fmt(tmp_graph, down, color_down)
        both = set(up).intersection(set(down))
        tmp_graph = add_color_graphviz_fmt(tmp_graph, both, 'yellow')

        if labels is not None:
            tmp_graph.graph['label'] = labels[n]
            tmp_graph.graph['fontsize'] = 13

        s_name = '%s_%04i.png' % (save_name, n)

        export_to_dot(tmp_graph, s_name, 'png', 'dot')
        draw_graphviz(tmp_graph, layout='dot', save_name=s_name,
                      image_format='png')
        string += s_name + ' '

    if create_gif:
        os.system('{}  {}.gif'.format(string, save_name))
        os.system('{}  {}.pdf'.format(string, save_name))


def run_from_ipython():
    try:
        __IPYTHON__
        return True
    except NameError:
        return False



import itertools

import networkx as nx
import pathos.multiprocessing as mp


def remove_unmeasured_nodes(graph, measured):
    new_g = graph.copy()
    edge_info_dict = dict()
    for i, j, data in graph.edges(data=True):
        edge_info_dict[(i, j)] = data
    nodes = set(graph.nodes())
    include = set(measured)
    include.intersection_update(nodes)

    # for i, j in :
    def find(d):
        i, j = d
        paths = []
        if nx.has_path(graph, i, j):
            for p in nx.all_shortest_paths(graph, i, j):
                path = []
                label = []
                for n in p:
                    if n in include:
                        path.append(n)
                    else:
                        label.append(n)
                if len(path) == 2:
                    paths.append((path, '|'.join(l for l in label)))
        return paths

    x = mp.Pool(4)
    paths = x.map(find, itertools.combinations(include, 2))
    to_remove = set()
    for p in paths:
        if len(p) != 2:
            continue
        for path, label in p:
            print(path, label)
            for n in label.split('|'):
                to_remove.add(n)
            new_g.add_edge(path[0], path[1], label=label)
    for n in to_remove:
        new_g.remove_node(n)
    return new_g


def merge_nodes(graph):
    """ merges nodes into single node if same neighbors

    Parameters
    ----------
    graph: nx.DiGraph

    Returns
    -------
    nx.DiGraph
    """
    neighbors2node = dict()

    nodes = set(graph.nodes())
    edges = set(graph.edges())

    new_g = graph.copy()
    for i in nodes:
        down = '|'.join(sorted(graph.successors(i)))
        up = '|'.join(sorted(graph.predecessors(i)))

        if (down, up) in neighbors2node:
            neighbors2node[(down, up)].add(i)
        else:
            neighbors2node[(down, up)] = {i}

    for node, neigh in neighbors2node.items():
        # print(node, neigh)
        if len(neigh) > 1:
            print(node, neigh)
            new_name = '|'.join(sorted(neigh))
            for x in node[0].split('|'):
                if x != '':
                    for n in neigh:
                        if (x, n) in edges:
                            new_g.remove_edge(x, n)
                            new_g.add_edge(x, new_name, **graph.edge[x][n])
                            edges.remove((x, n))

            for x in node[1].split('|'):
                if x != '':
                    for n in neigh:
                        if (n, x) in edges:
                            new_g.remove_edge(n, x)
                            new_g.add_edge(new_name, x, **graph.edge[n][x])
                            edges.remove((n, x))
    print("{} nodes and {} edges".format(len(graph.nodes), len(graph.edges)))
    print("{} nodes and {} edges".format(len(new_g.nodes), len(new_g.edges)))
    for n in new_g.nodes():
        if len(new_g.successors(n)) == 0 and len(new_g.predecessors(n)) == 0:
            new_g.remove_node(n)

    print("{} nodes and {} edges".format(len(new_g.nodes), len(new_g.edges)))
    return new_g


def compress_edges(graph):
    """
    compress edges of networks by finding common paths

    Parameters
    ----------
    graph: networkx.DiGraph

    Returns
    -------

    """

    g = graph.copy()
    nodes = set(g.nodes())
    neighbor_dict = {}
    for n in nodes:
        neigh = tuple(g.neighbors(n))
        if tuple(neigh) in neighbor_dict:
            neighbor_dict[tuple(neigh)].append(n)
        else:
            neighbor_dict[tuple(neigh)] = []
            neighbor_dict[tuple(neigh)].append(n)

    for i in neighbor_dict:
        neigh = neighbor_dict[i]
        # print(i,neigh)
        if len(i) != 1 or len(neigh) == 1:
            continue
        interaction_types = {}
        for j in neighbor_dict[i]:
            if g.has_edge(i[0], j):
                direction = 'type1'
                edge = g.get_edge(i[0], j)
            elif g.has_edge(j, i[0]):
                direction = 'type2'
                edge = g.get_edge(j, i[0])
            edge_type = edge.attr['arrowhead']
            if edge_type + direction in interaction_types:
                interaction_types[edge_type + direction].append(j)
            else:
                interaction_types[edge_type + direction] = []
                interaction_types[edge_type + direction].append(j)
        for k in interaction_types:
            if len(interaction_types[k]) > 1:
                # print(i[0],'->',)
                to_join = []
                for each in interaction_types[k]:
                    if len(g.neighbors(each)) == 1:
                        to_join.append(each)
                # print(to_join,k)
                if len(to_join) > 1:
                    label = "{"
                    for node in to_join:
                        g.remove_node(node)
                        label += ' %s |' % str(node)
                    label = label[:-1]
                    label += "}"
                    # print(label)
                    g.add_node(label, shape='record', label=label)
                    if k.endswith('type2'):
                        g.add_edge(label, i[0], dir='both', arrowhead=k[:-5],
                                   arrowtail="none")
                    elif k.endswith('type1'):
                        g.add_edge(i[0], label, dir='both', arrowhead=k[:-5],
                                   arrowtail="none")
    return g



"""Interface to downloading ID mapping databases.

.. topic:: Databases supported

    :URL: https://www.uniprot.org/
    :URL: https://www.ncbi.nlm.nih.gov/
    :URL: http://www.hmdb.ca/

"""

from .download_libraries import HMDB, download_hgnc, download_ncbi, \
    download_uniprot, load_hgnc, load_ncbi, load_uniprot

__all__ = ['load_hgnc', 'load_ncbi', 'load_uniprot', 'HMDB']



import logging
import math
import os
import zipfile

from defusedxml import cElementTree as ElementTree
import pandas as pd
import requests

from magine.data.storage import id_mapping_dir
from magine.logging import get_logger

logger = get_logger(__name__, log_level=logging.INFO)


def load_hgnc():
    hgnc_name = os.path.join(id_mapping_dir, 'hgnc.gz')
    if not os.path.exists(hgnc_name):
        hgnc = download_hgnc()
        _check_path(hgnc_name)
    else:
        hgnc = pd.read_csv(hgnc_name, low_memory=False)

    return hgnc.loc[hgnc.status == 'Approved']


def load_uniprot():
    # gather data from uniprot
    uniprot_path = os.path.join(id_mapping_dir, 'human_uniprot.csv.gz')
    if not os.path.exists(uniprot_path):
        uniprot = download_uniprot()
        _check_path(uniprot_path)
    else:
        uniprot = pd.read_csv(uniprot_path, low_memory=False)
    return uniprot


def load_ncbi():
    ncbi_name = os.path.join(id_mapping_dir, 'ncbi.gz')
    if not os.path.exists(ncbi_name):
        ncbi = download_ncbi()
        _check_path(ncbi_name)
    else:
        ncbi = pd.read_csv(ncbi_name, low_memory=False)
    return ncbi


def _check_path(path):
    if not os.path.exists(path):
        raise AssertionError()


def download_uniprot(species='hsa'):
    """
    `<https://www.uniprot.org/>`_

    Parameters
    ----------
    species : str
        Species name. Currently only human is supported.
        Please let us know if you need other species


    """
    _url_h = 'ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/by_organism/{}'

    if species == 'hsa':
        url = _url_h.format('HUMAN_9606_idmapping.dat.gz')
    else:
        raise ValueError("Currently only implemented human uniprot. "
                         "Please contact us for additional databases")

    columns = ['uniprot', 'mapping_type', 'mapping']

    df = _download(url, columns, name='uniprot', save=False, names=columns,
                   compression='gzip')

    uniprot = pd.pivot_table(df, columns='mapping_type', index='uniprot',
                             aggfunc='first')

    uniprot.columns = uniprot.columns.droplevel()
    uniprot.reset_index(inplace=True)
    outfile = os.path.join(id_mapping_dir, 'human_uniprot.csv.gz')
    uniprot.to_csv(outfile, compression='gzip',
                   header=True, index=False)

    return uniprot


def download_hgnc():
    """
    Downloads HGNC and stores it as a pandas.DataFrame
    `<http://www.genenames.org/>`_

    """

    url = 'ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/locus_groups/protein-coding_gene.txt'

    columns = ['symbol', 'uniprot_ids', 'ensembl_gene_id', 'name', 'location',
               'entrez_id', 'ucsc_id', 'vega_id', 'alias_name', 'alias_symbol',
               'status', 'gene_family', 'gene_family_id', 'ena', 'iuphar',
               'cd',
               'refseq_accession', 'ccds_id', 'pubmed_id', 'mgd_id', 'rgd_id',
               'lsdb', 'bioparadigms_slc', 'enzyme_id', 'merops', 'horde_id',
               'pseudogene.org', 'cosmic', 'rna_central_ids', 'omim_id',
               'imgt',
               'intermediate_filament_db',
               ]
    return _download(url, columns, 'hgnc')


def download_ncbi():
    """
    Downloads data from NCBI

    `<https://www.ncbi.nlm.nih.gov/>`_

    """
    url = 'ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/GENE_INFO/Mammalia/Homo_sapiens.gene_info.gz'

    columns = ['GeneID', 'Symbol', 'description']
    return _download(url, columns, 'ncbi', compression='gzip')


def _download(url, columns, name, save=True, names=None, compression=None):
    logger.info("Downloading from {}.".format(name.upper()))
    if names is None:
        df = pd.read_table(url, delimiter='\t', low_memory=False, verbose=True,
                           compression=compression)
    else:
        df = pd.read_table(url, delimiter='\t', low_memory=False, verbose=True,
                           names=names, compression=compression)
    df = df[columns]
    if save:
        outfile = os.path.join(id_mapping_dir, '{}.gz'.format(name))
        df.to_csv(outfile, compression='gzip', header=True, index=False)
    logger.info("Done downloading from {}.".format(name.upper()))
    return df


def download_hmdb():
    """
    Downloads data from HMDB

    `<http://www.hmdb.ca/>`_

    """
    HMDB().download_db(fresh_download=True)


class HMDB(object):
    """
    Downloads and processes HMDB metabolites database

    `<http://www.hmdb.ca/>`_

    """

    def __init__(self):
        self.id_dir = id_mapping_dir
        self.target_file = 'hmdb_metabolites.zip'
        self.out_name = os.path.join(self.id_dir, 'hmdb_dataframe.csv.gz')
        self.hmdb_file = os.path.join(self.id_dir, self.target_file)

    def load_db(self, fresh_download=False):
        if not os.path.exists(self.out_name) or fresh_download:
            self.download_db(fresh_download)
        return pd.read_csv(self.out_name, low_memory=False, encoding='utf-8')

    def download_db(self, fresh_download):
        """ parse HMDB to Pandas.DataFrame

        """
        out_dir = os.path.join(self.id_dir, 'HMDB')
        # create output directory
        if not os.path.exists(out_dir):
            os.mkdir(out_dir)

        # download and unzip xml file
        if fresh_download or not len(os.listdir(out_dir)):
            self._download_hmdb()
            # unzips hmdb metabolites file
            logger.info("Unzipping metabolites file")
            zip_ref = zipfile.ZipFile(self.hmdb_file, 'r')
            zip_ref.extractall(out_dir)
            zip_ref.close()
            logger.info("Done unzipping metabolites file")

        # creates dataframe
        logger.info("Parsing metabolites information from files")
        df = pd.DataFrame([
            self._create_dict(e) for ev, e in
            iter(ElementTree.iterparse(
                os.path.join(out_dir, os.listdir(out_dir)[0]),
                events=("start", "end"))
            )
            if e.tag == '{http://www.hmdb.ca}metabolite' and ev == 'end'
        ])

        for i in categories:
            if i in df.columns:
                df[i.split('}')[1]] = df[i]
                del df[i]

        df.to_csv(self.out_name, index=False, encoding='utf-8',
                  compression='gzip')
        logger.info("Done processing HMDB")

    def _download_hmdb(self):
        """ Downloads hmdb metabolites xml file """
        logger.info("Downloading metabolites information from HMDB")
        ur = 'http://www.hmdb.ca/system/downloads/current/hmdb_metabolites.zip'

        r = requests.get(ur, stream=True)
        file_size = int(r.headers['content-length'])
        logger.info("File size is {} bytes".format(file_size))
        file_size_dl = 0
        block_sz = 1024
        # block_sz = 8024
        v = set()
        milestone_markers = range(0, 101, 10)

        with open(self.hmdb_file, 'wb') as f:
            for chunk in r.iter_content(chunk_size=block_sz):
                file_size_dl += len(chunk)
                percent_done = int(math.floor(file_size_dl * 100. / file_size))
                if percent_done in milestone_markers:
                    if percent_done not in v:
                        logger.info("{}%".format(percent_done))
                        v.add(percent_done)
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)

        logger.info("Done downloading {}.".format(ur))

    @staticmethod
    def _create_dict(elem):
        template = {}
        for i in categories:
            n = elem.find(i)
            if n is None:
                # print(i)
                continue
            elif i == '{http://www.hmdb.ca}protein_associations':
                output = []
                for pr in n.findall('{http://www.hmdb.ca}protein'):
                    for gn in pr.findall('{http://www.hmdb.ca}gene_name'):
                        if gn.text is not None:
                            output.append(gn.text)
                template[i] = '|'.join(output)

            elif i == '{http://www.hmdb.ca}synonyms':
                template[i] = '|'.join(
                    [p.text for p in n.findall('{http://www.hmdb.ca}synonym')]
                )

            elif i == '{http://www.hmdb.ca}secondary_accessions':
                accession = n.findall('{http://www.hmdb.ca}accession')
                if len(accession) == 0:
                    accession = ''
                else:
                    accession = '|'.join(sorted([a.text for a in accession]))
                template[i] = accession

            else:
                template[i] = n.text
        elem.clear()
        return template


categories = ['kegg_id', 'name', 'accession', 'chebi_id',
              'chemspider_id', 'biocyc_id', 'synonyms',
              'pubchem_compound_id', 'protein_associations',
              'inchikey', 'iupac_name', 'drugbank_id',
              'chemical_formula', 'smiles', 'metlin_id',
              'average_molecular_weight', 'secondary_accessions',
              # 'normal_concentrations',
              # 'molecular_framework'
              #  'pathways',
              ]

categories = ['{http://www.hmdb.ca}' + i for i in categories]

valid_uniprot_cols = ['uniprot', 'Allergome', 'BioCyc', 'BioGrid', 'BioMuta',
                      'CCDS', 'CRC64', 'ChEMBL', 'ChiTaRS', 'CleanEx', 'DIP',
                      'DMDM', 'DNASU', 'DisProt', 'DrugBank', 'EMBL',
                      'EMBL-CDS', 'ESTHER', 'Ensembl', 'Ensembl_PRO',
                      'Ensembl_TRS', 'GI', 'GeneCards','GeneDB', 'GeneID',
                      'GeneReviews', 'GeneTree', 'GeneWiki', 'Gene_Name',
                      'Gene_ORFName', 'Gene_Synonym', 'GenomeRNAi',
                      'GuidetoPHARMACOLOGY', 'H-InvDB', 'HGNC', 'HOGENOM',
                      'HOVERGEN', 'HPA', 'KEGG', 'KO', 'MEROPS', 'MIM', 'MINT',
                      'NCBI_TaxID', 'OMA', 'Orphanet', 'OrthoDB', 'PATRIC',
                      'PDB', 'PeroxiBase', 'PharmGKB', 'REBASE', 'Reactome',
                      'RefSeq', 'RefSeq_NT', 'STRING', 'SwissLipids', 'TCDB',
                      'TreeFam', 'UCSC', 'UniGene', 'UniParc', 'UniPathway',
                      'UniProtKB-ID', 'UniRef100', 'UniRef50', 'UniRef90',
                      'eggNOG', 'neXtProt']

ncbi_valid_categories = ['GeneID', 'Symbol', 'description']

hgnc_valid_categories = ['symbol', 'uniprot_ids', 'ensembl_gene_id', 'name',
                         'location', 'entrez_id', 'ucsc_id', 'vega_id',
                         'alias_name', 'alias_symbol', 'status', 'gene_family',
                         'gene_family_id', 'ena', 'iuphar', 'cd',
                         'refseq_accession', 'ccds_id', 'pubmed_id', 'mgd_id',
                         'rgd_id', 'lsdb', 'bioparadigms_slc', 'enzyme_id',
                         'merops', 'horde_id', 'pseudogene.org', 'cosmic',
                         'rna_central_ids', 'omim_id', 'imgt',
                         'intermediate_filament_db'
                         ]

if __name__ == '__main__':
    # download_hgnc()
    download_uniprot('hsa')
    # hmdb = HMDB()
    # df = hmdb.load_db(fresh_download=True)
    # print(df.head(10))
    # print(df.columns)



import os

import matplotlib.figure
import matplotlib.pyplot as plt
from nose.tools import ok_, raises

import magine.enrichment.enrichment_result as et
from magine.data.experimental_data import load_data
from magine.plotting.heatmaps import heatmap_by_terms

data_dir = os.path.dirname(__file__)


class TestEnrichmentResult(object):
    def setUp(self):
        d_name = os.path.join(data_dir, 'Data', 'enrichr_test_enrichr.csv')
        self.data = et.load_enrichment_csv(d_name)

    def test_filter_row(self):
        terms = ['p53 signaling pathway_hsa_hsa04115',
                 'apoptosis_hsa_hsa04210'],

        # checks if single entry
        slimmed = self.data.filter_rows('term_name', terms[0])
        ok_(slimmed.shape[0] == 6)

        # checks if list
        slimmed = self.data.filter_rows('term_name', terms)
        ok_(slimmed.shape[0] == 128)

    def test_filter_multi(self):
        slimmed = self.data.filter_multi(p_value=0.05, combined_score=20)
        ok_(slimmed.shape[0] == 8)

    def test_term_to_gene(self):
        genes = self.data.term_to_genes('apoptosis_hsa_hsa04210')
        ok_(genes == {'CASP8', 'CASP10', 'BCL2', 'BAX', 'CASP3'})

    def test_filter_based_on_word(self):
        slimmed = self.data.filter_based_on_words('p53')
        ok_(slimmed.shape[0] == 5)

        slimmed = self.data.filter_based_on_words(['apop', 'p53'])
        ok_(slimmed.shape[0] == 11)

    def test_all_genes(self):
        all_g = self.data.all_genes_from_df()
        ok_(all_g == {'CASP8', 'CASP10', 'BCL2', 'BAX', 'CASP3'})

    def test_filter_sim_terms(self):
        sim2 = self.data.remove_redundant(level='sample', sort_by='rank')
        ok_(sim2.shape[0] == 7)

        sim2 = self.data.remove_redundant(level='sample')
        ok_(sim2.shape[0] == 7)

        sim2 = self.data.remove_redundant(level='dataframe', verbose=True)
        ok_(sim2.shape[0] == 15)

        copy_data = self.data.copy()
        copy_data.remove_redundant(level='sample', verbose=True, inplace=True)
        ok_(copy_data.shape[0] == 7)

    def test_dist(self):
        # dist = self.data.dist_matrix()
        # assert isinstance(dist, matplotlib.figure.Figure)
        copy_data = self.data.copy()
        copy_data.remove_redundant(level='sample', verbose=False, inplace=True)
        copy_data.dist_matrix()

        copy_data.heatmap(convert_to_log=True, index='term_name',
                          columns='sample_id', figsize=(6, 14),
                          annotate_sig=True, linewidths=.01, cluster_row=True)
        plt.close()
        dist = self.data.dist_matrix(figsize=(3, 3), level='each')
        ok_(isinstance(dist, matplotlib.figure.Figure))
        plt.close()

        df = load_data(os.path.join(os.path.dirname(__file__), 'Data',
                                    'example_apoptosis.csv'))
        terms = [{'BAX'}, {'PARP4'}]
        heatmap_by_terms(df.species, ['1', '2'], terms)
        plt.close()

    @raises(DeprecationWarning)
    def test_deprecated_warning(self):
        # dist = self.data.dist_matrix()
        # assert isinstance(dist, matplotlib.figure.Figure)
        copy_data = self.data.copy()
        copy_data.remove_redundant(level='sample', verbose=False, inplace=True)

        copy_data.heatmap(convert_to_log=True, index='term_name',
                          columns='sample_id', figsize=(6, 14),
                          rank_index=True,
                          )
        plt.close()

    def test_find_similar_terms(self):
        sim = self.data.find_similar_terms('apoptotic process')
        print(sim)

    def test_jaccard_index(self):
        term1 = {'BAX', 'BCL2', 'MCL1', 'CASP3'}
        term2 = {'BAX', 'BCL2', 'MCL1', 'TP53'}
        score = et.jaccard_index(term1, term2)

        ok_(score == 0.6)

    def test_term_to_dict(self):
        g = self.data.term_to_genes_dict(
            ['apoptosis_hsa_hsa04210', 'p53 signaling pathway_hsa_hsa04115']
        )

        ok_(g['apoptosis_hsa_hsa04210'] == {'BCL2', 'CASP10'})
        ok_(g['apoptosis_hsa_hsa04210,p53 signaling pathway_hsa_hsa04115'] == {
            'CASP3', 'CASP8', 'BAX'})



import os
from shutil import rmtree

from appdirs import user_data_dir

dir_name = user_data_dir('magine')

dir_name = os.getenv('MAGINE_DATA', dir_name)

network_data_dir = os.path.join(dir_name, 'network_data')
id_mapping_dir = os.path.join(dir_name, 'id_data')


def create_storage_structure():
    # check or create the main data storage directory
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)

    # check or create the id mapping data storage directory
    if not os.path.exists(id_mapping_dir):
        os.makedirs(id_mapping_dir)

    # check or create the network data storage directory
    if not os.path.exists(network_data_dir):
        os.makedirs(network_data_dir)


def clear_cached_dbs():
    """Remove old database cached downloads"""
    # check or create the main data storage directory
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    # check or create the network data storage directory
    if os.path.exists(id_mapping_dir):
        rmtree(id_mapping_dir)

    # check or create the network data storage directory
    if os.path.exists(network_data_dir):
        rmtree(network_data_dir)


create_storage_structure()



import networkx as nx
from nose.tools import ok_

from magine.mappings.chemical_mapper import ChemicalMapper
from magine.mappings.gene_mapper import GeneMapper

cm = ChemicalMapper()
gm = GeneMapper()


class TestChemicalMapper(object):

    def test_synonyms(self):
        hmdb = cm.check_synonym_dict(term='dodecene',
                                     format_name='main_accession')

        ok_((hmdb == ['HMDB0000933', 'HMDB0059874']))

    def test_protein_network(self):
        item = 'HMDB42489'
        hit_list = ['PNLIP', 'LIPC', 'LIPA', 'PNLIPRP1', 'PNPLA3', 'LIPF',
                    'LIPG', 'CEL', 'DGAT1', 'PNLIPRP2', 'CPT1B', 'CPT1A',
                    'LPL', 'CPT2', 'MGLL', 'CES1', 'LIPE', 'MTTP', 'APOA1',
                    'CETP', 'APOE', 'APOC3', 'APOB', 'APOA4', 'CD36', 'P4HB',
                    'MOGAT2', 'PNPLA4', 'SLC27A1', 'DGAT2', 'MOGAT1',
                    'MOGAT3', 'PLB1', 'APOA5']

        for protein in cm.hmdb_to_protein[item]:
            ok_(protein in hit_list)


class TestGeneMapper(object):
    def test_kegg_to_gene_name(self):
        ok_(gm.kegg_to_gene_name['hsa:581'][0] == 'BAX')

    def test_kegg_to_uniprot(self):
        ok_(gm.kegg_to_uniprot['hsa:581'][0] == 'Q07812')

    def test_gene_name_to_ensemble(self):
        ok_(gm.gene_name_to_ensembl['BAX'][0] == 'ENSG00000087088')

    def test_gene_name_to_uniprot(self):
        ok_(gm.gene_name_to_uniprot['BAX'][0] == 'Q07812')

    def test_uniprot_to_gene_name(self):
        ok_(gm.uniprot_to_gene_name['Q07812'][0] == 'BAX')


def test_kegg_to_hmdb():
    """
    tests kegg compound to hmdb
    :return:
    """
    g = nx.DiGraph()
    g.add_edge('hsa:224', 'hsa:219')
    g.add_edge('hsa:219', 'cpd:C00197')
    g.add_edge('cpd:C00197', 'cpd:C00197')
    g.add_edge('cpd:C15972', 'cpd:C00197')
    g.add_edge('cpd:C15972', 'cpd:C00469')

    change_dict, kegg_short, chem_names = cm.convert_kegg_nodes(g)
    nx.set_node_attributes(g, kegg_short, 'keggName')
    nx.set_node_attributes(g, chem_names, 'chemName')
    nx.set_node_attributes(g, change_dict, 'hmdbNames')

    dict2, kegg_short = gm.convert_kegg_nodes(g)
    nx.set_node_attributes(g, kegg_short, 'keggName')
    change_dict.update(dict2)
    g = nx.relabel_nodes(g, change_dict)
    answer = '(2R)-2-Hydroxy-3-(phosphonatooxy)propanoate'
    ok_(g.node['HMDB0060180']['chemName'] == answer)
    ok_(g.node['HMDB0060180']['keggName'] == 'C00197')


def test_kegg_to_uniprot():
    """
    tests kegg gene to gene name
    :return:
    """
    g = nx.DiGraph()
    g.add_edge('hsa:217', 'hsa:219')
    g.add_edge('hsa:217', 'hsa:223')
    g.add_edge('hsa:501', 'hsa:219')
    g.add_edge('hsa:224', 'hsa:219')
    g.add_node('hsa:857')
    dic, kegg_short = gm.convert_kegg_nodes(g, species='hsa')
    nx.set_node_attributes(g, kegg_short, 'keggName')
    g = nx.relabel_nodes(g, dic)

    valid = {
        'ALDH2': '217',
        'ALDH1B1': '219',
        'ALDH9A1': '223',
        'ALDH7A1': '501',
        'ALDH3A2': '224',
        'CAV1': '857'
    }
    for k, v in valid.items():
        ok_(g.node[k]['keggName'] == v)


if __name__ == '__main__':
    test_kegg_to_hmdb()



import itertools
import os
import random
from itertools import combinations, product

import networkx as nx
import numpy as np
import pandas as pd
import scipy.stats as stats
from statsmodels.stats.multitest import fdrcorrection

import magine.networks.utils as nt
from magine.networks.visualization import draw_graphviz, draw_igraph


class AnnotatedSetNetworkGenerator(object):
    """ Generates ontology network from molecular networks.

    Nodes are the ontology terms
    Edges between nodes are determined based on molecular network graphs.

    """

    def __init__(self, network):
        """

        Parameters
        ----------
        network : nx.DiGraph
        """
        # make sure a network exists
        if not(isinstance(network, nx.DiGraph)):
            raise Exception("Must provide a nx.DiGraph")
        self.network = network.copy()
        # remove all self loops from graph
        self.network.remove_edges_from(nx.selfloop_edges(self.network))
        self.nodes = set(self.network.nodes)
        self.edges = set(self.network.edges)
        self.molecular_network = None

    def create_network_from_list(self, list_of_ontology_terms,
                                 ont_to_species_dict, ont_to_label_dict,
                                 save_name=None, draw=False, out_dir=None,
                                 use_threshold=True, use_fdr=False,
                                 min_edges=0):
        """
        Creates a GO level network from list of GO terms

        Parameters
        ----------

        list_of_ontology_terms : list_list
            list of GO terms
        ont_to_species_dict : dict
            dictionary, where keys are ontology terms and values are species
            for that term
        ont_to_label_dict: dict
            dictionary where keys are ontology terms and values are labels
        save_name : str
            name to save network
        draw : bool
            create a go_graph of network
        use_threshold : bool
            Use binomial test to check for edge significance
            Uses bh correction for multiple hypothesis testing
        out_dir : str
            output directory
        use_fdr : bool
            Use FDR correction for edge
        min_edges : int
            Minimum number of edges between terms

        Returns
        -------
        asn : networkx.DiGraph
            Annotated set network
        mol_net : networkx.DiGraph
            Sub-network made my molecular level species

        """
        if out_dir is not None:
            out_path = os.path.join(out_dir, 'Network_files')
            if not os.path.exists(out_dir):
                os.mkdir(out_dir)
                os.mkdir(out_path)
        else:
            out_path = '.'

        list_of_go_terms = set(list_of_ontology_terms)

        dd = nx.out_degree_centrality(self.network)
        x = 0
        for _, i in dd.items():
            x += i
        avg_conn = x / len(dd)
        # n_total_edges = len(self.edges)
        p_values = []
        # create dictionaries that add the label and terms as node attributes
        gene_to_term, gene_to_label = dict(), dict()

        def add_to_dict(genes, term):

            for gene in genes:
                if gene in gene_to_term:
                    gene_to_term[gene].add(term)
                    gene_to_label[gene].add(ont_to_label_dict[term])
                else:
                    gene_to_term[gene] = {term}
                    gene_to_label[gene] = {ont_to_label_dict[term]}

        def get_edges(set1, set2):
            possible_edges = set(product(set1, set2.difference(set1)))

            n_possible = len(possible_edges)
            edge_hits = possible_edges.intersection(self.edges)
            n_hits = len(edge_hits)
            # odds_to_find = float(n_possible) / n_total_edges
            # p_val = stats.binom_test(n_hits, n_possible, odds_to_find)
            # print(stats.binom_test(n_hits, n_possible, odds_to_find),
            #       stats.binom_test(n_hits, n_possible, avg_conn))
            p_val = stats.binom_test(n_hits, n_possible, avg_conn)
            return [edge_hits, n_hits, p_val]

        for term1, term2 in combinations(list_of_go_terms, 2):
            term_1 = set(ont_to_species_dict[term1]).intersection(self.nodes)
            term_2 = set(ont_to_species_dict[term2]).intersection(self.nodes)

            add_to_dict(term_1, term1)
            add_to_dict(term_2, term2)
            # print(get_edges(term_1, term_2)[1:],
            # gather_stats(term_1, term_2, self.nodes, self.edges)[1:])

            # p_values.append(
            #     [term1, term2] + gather_stats(term_1, term_2, self.nodes,
            #                                   self.edges))
            # p_values.append(
            #     [term2, term1] + gather_stats(term_2, term_1, self.nodes,
            #                                   self.edges))

            p_values.append([term1, term2] + get_edges(term_1, term_2))
            p_values.append([term2, term1] + get_edges(term_2, term_1))

        cols = ['term1', 'term2', 'edges', 'n_edges', 'p_value', ]

        df = pd.DataFrame(p_values, columns=cols)
        df = df.loc[df['n_edges'] > min_edges].copy()
        # FDR correction
        _, df['adj_p_value'] = fdrcorrection(df['p_value'])

        if use_threshold:
            if use_fdr:
                df = df.loc[df['adj_p_value'] <= .05]
            else:
                df = df.loc[df['p_value'] <= .05]
        cols += ['adj_p_value']
        # create empty networks
        asn = nx.DiGraph()
        mol_net = nx.DiGraph()
        for term1, term2, edge, n_edges, p_value, adj_pval in df[cols].values:

            label_1 = ont_to_label_dict[term1]
            label_2 = ont_to_label_dict[term2]
            asn.add_node(label_1, term=term1, label=label_1)
            asn.add_node(label_2, term=term2, label=label_2)
            asn.add_edge(label_1, label_2, label=str(n_edges), weight=n_edges,
                         pvalue=p_value, adjPval=adj_pval)

            nodes = list(itertools.chain(*edge))

            nodes += ont_to_species_dict[term1]
            nodes += ont_to_species_dict[term2]

            x = self.network.subgraph(nodes).copy()

            mol_net.add_edges_from(list(x.edges(data=True)))

        # addes missing edges between that maybe absent due to threshold, want
        # to keep in network
        for i, j, d in self.network.subgraph(mol_net.nodes).edges(data=True):
            mol_net.add_edge(i, j, **d)

        for i in mol_net.nodes:
            labels = gene_to_label[i]
            terms = gene_to_term[i]
            if len(labels) != len(terms):
                raise AssertionError('len(labels) should equal len(terms)')
            mol_net.node[i]['termName'] = ','.join(sorted(labels))
            mol_net.node[i]['terms'] = ','.join(sorted(terms))

        self.molecular_network = mol_net
        if save_name is not None:
            if out_dir is not None:
                save_name = os.path.join(out_path, save_name)
            nx.write_gml(mol_net, '{}_subgraph.gml'.format(save_name))
            nx.write_gml(asn, '{}.gml'.format(save_name))

            if draw:
                draw_graphviz(asn, save_name=save_name)
                draw_igraph(mol_net, save_name + '_subgraph_igraph')
        return asn, mol_net


def gather_stats(set1, set2, backgroud_nodes, edges):
    n_set1 = len(set1)
    n_set2 = len(set2.difference(set1))
    total_nodes = n_set1 + n_set2

    n_tests = 100
    odds = np.zeros(n_tests)
    nodes_list = list(backgroud_nodes)
    n_possible = range(len(backgroud_nodes))
    for i in range(n_tests):
        s = [nodes_list[j] for j in random.sample(n_possible, total_nodes)]
        odds[i] = len(set(product(s[:n_set1], s[n_set1:])).intersection(edges))

    # calculates edges between two sets
    possible_edges = set(product(set1, set2.difference(set1)))
    edge_hits = possible_edges.intersection(edges)
    n_hits = len(edge_hits)

    # sort to find probability of finding random sets connectivity
    odds.sort()
    first_index = np.searchsorted(odds, n_hits, side='right')
    probability = float(n_tests - first_index) / n_tests
    p_val = stats.binom_test(n_hits, len(possible_edges), probability)

    return [edge_hits, len(edge_hits), p_val]


def create_subnetwork(df, network, terms=None, save_name=None, draw_png=False,
                      remove_isolated=False, use_cytoscape=False, merge=False,
                      out_dir=None, use_threshold=False, use_fdr=False,
                      min_edges=0):
    """

    Parameters
    ----------

    df : pd.DataFrame
    network : nx.DiGraph
    terms : list, optional
        List of terms to use in ont network. Default is all terms
    save_name : str
    draw_png : bool
    remove_isolated : bool
        Remove nodes that are not connected in the final graphs
    use_cytoscape : bool
        Create ontology network only, don't visualize with cytoscape
    merge : bool
        Create gif of cytoscape session
    out_dir : str
        Output directory of images
    use_threshold : bool
        Use binomial test to calculate edge relevance between terms
    use_fdr : bool
        Use BH correction on binomial test
    min_edges : int
        Minimum number of edges between two terms

    Returns
    -------

    """
    if terms is not None:
        df_copy = df[df['term_name'].isin(terms)].copy()
        terms = set(terms)
    else:
        df_copy = df.copy()
        terms = set(df['term_name'].values)

    # normalize enriched scores
    df_copy['combined_score'] = np.abs(df_copy['combined_score'])
    # df_copy['combined_score'] = np.log2(df_copy['combined_score'])
    # df_copy.loc[df['combined_score'] > 150, 'combined_score'] = 150

    labels = df_copy['sample_id'].unique()
    # create dictionary of values
    label_dict, term_dict = dict(), dict()
    for i in terms:
        genes = set(df_copy.term_to_genes(i))
        term_dict[i] = genes
        label_dict[i] = i

    ong = AnnotatedSetNetworkGenerator(network=network)

    print("Creating ontology network")
    term_g, molecular_g = ong.create_network_from_list(
        terms, term_dict, label_dict, save_name=save_name, draw=draw_png,
        use_threshold=use_threshold, use_fdr=use_fdr, min_edges=min_edges
    )

    if remove_isolated:
        nt.remove_isolated_nodes(term_g)
        nt.remove_isolated_nodes(molecular_g)

    score_array = pd.pivot_table(df_copy, index=['term_name'],
                                 columns='sample_id')
    x = score_array['combined_score'].fillna(0)

    for i in term_g.nodes:
        values = x.loc[i]
        term_g.node[i]['color'] = 'red'
        term_g.node[i]['label'] = i
        for n, time in enumerate(labels):
            fmt_label = 'sample{}'.format(time.replace('_', ''))
            term_g.node[i][fmt_label] = float(values[time])
    labels = [i.replace('_', '') for i in labels]
    if save_name:
        nx.write_graphml(term_g, "{}_ags_network.graphml".format(save_name))
    if use_cytoscape:
        from magine.networks.visualization.cytoscape import RenderModel
        rm = RenderModel(term_g, layout='force-directed')
        rm.visualize_by_list_of_time(labels,
                                     prefix=save_name,
                                     out_dir=out_dir,
                                     )
        _s = "convert -delay 100 -dispose previous -loop 0 " \
             "ont_network_*_formatted.png {}.gif"
        if merge:
            os.system(_s.format(save_name))
    return term_g, molecular_g



import logging
import os

import networkx as nx

from magine.data.storage import network_data_dir
from magine.logging import get_logger

logger = get_logger(__name__, log_level=logging.INFO)


def load_hmdb_network(fresh_download=False):
    """ Create HMDB network containing all metabolite-protein interactions

    Parameters
    ----------
    fresh_download : bool
        Download fresh copy from HMDB
    verbose : bool

    Returns
    -------
    nx.DiGraph
    """
    out_name = os.path.join(network_data_dir, 'hmdb_graph.p.gz')

    if not fresh_download and os.path.exists(out_name):
        tmp_graph = nx.read_gpickle(out_name)
    else:
        from magine.mappings.chemical_mapper import ChemicalMapper

        cm = ChemicalMapper()

        tmp_graph = nx.DiGraph()

        def _add_node(node, node_type):
            attrs = {'databaseSource': 'HMDB', 'speciesType': node_type}
            if node_type == 'compound':
                if node in cm.hmdb_to_chem_name:
                    attrs['chemName'] = sorted(cm.hmdb_to_chem_name[node])[0]
            tmp_graph.add_node(node, **attrs)

        for source, genes in cm.hmdb_main_to_protein.items():
            if source == '':
                continue
            _add_node(source, 'compound')
            for target in genes:
                if target == '':
                    continue
                _add_node(target, 'gene')
                tmp_graph.add_edge(source, target, interactionType='chemical',
                                   databaseSource='HMDB')
        nx.write_gpickle(tmp_graph, out_name)

    logger.info("HMDB : {} nodes and {} edges".format(
        len(tmp_graph.nodes), len(tmp_graph.edges))
    )

    return tmp_graph


if __name__ == '__main__':
    load_hmdb_network(True, True)



from bioservices import UniChem
import pandas as pd
from sortedcontainers import SortedSet, SortedDict

from magine.mappings.databases.download_libraries import HMDB

try:
    import cPickle as pickle
except:  # python3 doesnt have cPickle
    import pickle
try:
    basestring
# Allows isinstance(foo, basestring) to work in Python 3
except:
    basestring = str
chem = UniChem()


class ChemicalMapper(object):
    """ Convert chemical species across various ids.

    Database was creating using HMDB

    """

    valid_columns = ['kegg_id', 'name', 'accession', 'chebi_id', 'inchikey',
                     'chemspider_id', 'biocyc_id', 'synonyms', 'iupac_name',
                     'pubchem_compound_id', 'protein_associations',
                     'ontology', 'drugbank_id', 'chemical_formula',
                     'smiles', 'metlin_id', 'average_molecular_weight',
                     'secondary_accessions'
                     ]

    def __init__(self, fresh_download=False):
        """

        Parameters
        ----------
        fresh_download: bool
            download new copy of database

        """

        self.database = None
        self._hmdb_to_chem_name = None
        self._chem_name_to_hmdb = None
        self._hmdb_to_kegg = None
        self._kegg_to_hmdb = None
        self.synonyms_to_hmdb = None
        self._drugbank_to_hmdb = None
        self._hmdb_to_protein = None
        self._hmdb_main_to_protein = None
        self._hmdb_accession_to_main = None
        hmdb_database = HMDB().load_db(fresh_download=fresh_download)
        self.database = hmdb_database.where((pd.notnull(hmdb_database)), None)
        self.database['main_accession'] = self.database['accession']
        sub_db = self.database[
            self.database['secondary_accessions'].str.contains('|', na=False)]
        new_df = tidy_split(sub_db, 'secondary_accessions', '|')
        new_df['accession'] = new_df['secondary_accessions']
        self.database = pd.concat([self.database, new_df])
        self.kegg_hmdb = chem.get_mapping("kegg_ligand", "hmdb")
        self.kegg_to_hmdb = self._to_dict("kegg_id", "main_accession")
        self.hmdb_to_chem_name = self._to_dict("main_accession", "name")

    @property
    def hmdb_to_kegg(self):
        if self._hmdb_to_kegg is None:
            self._hmdb_to_kegg = self._to_dict("accession", "kegg_id")
        return self._hmdb_to_kegg

    @property
    def chem_name_to_hmdb(self):
        if self._chem_name_to_hmdb is None:
            self._chem_name_to_hmdb = self._to_dict("name", "main_accession")
        return self._chem_name_to_hmdb

    @property
    def drugbank_to_hmdb(self):
        if self._drugbank_to_hmdb is None:
            self._drugbank_to_hmdb = self._to_dict("drugbank_id",
                                                   "main_accession")
        return self._drugbank_to_hmdb

    @property
    def hmdb_to_protein(self):
        if self._hmdb_to_protein is None:
            self._hmdb_to_protein = self._from_list_dict(
                "accession", "protein_associations"
            )
        return self._hmdb_to_protein

    @property
    def hmdb_main_to_protein(self):
        if self._hmdb_main_to_protein is None:
            self._hmdb_main_to_protein = self._from_list_dict(
                "main_accession", "protein_associations"
            )
        return self._hmdb_main_to_protein

    @property
    def hmdb_accession_to_main(self):
        if self._hmdb_accession_to_main is None:
            self._hmdb_accession_to_main = self._from_list_dict(
                "accession", "main_accession"
            )
        return self._hmdb_accession_to_main

    def _to_dict(self, key, value):
        """ creates a dictionary with a list of values for each key

        Parameters
        ----------
        key : str
        value : str

        Returns
        -------
        dict

        """
        d = self.database[[key, value]].copy()
        d.dropna(how='any', inplace=True)
        return_dict = SortedDict()
        for i, j in d.values:
            i = i.strip()
            if i not in return_dict:
                return_dict[i] = set()
            return_dict[i].add(j)
        return return_dict

    def _from_list_dict(self, key, value):
        d = self.database[[key, value]].copy()
        d.dropna(how='any', inplace=True)
        return_dict = SortedDict()
        for i, j in d.values:
            if i in return_dict:
                return_dict[i].update(j.split('|'))
            else:
                return_dict[i] = SortedSet(j.split('|'))
        return return_dict

    def check_synonym_dict(self, term, format_name):
        """ checks hmdb database for synonyms and returns formatted name

        Parameters
        ----------
        term : str
        format_name : str

        Returns
        -------
        dict


        Examples
        --------
        >>> cm = ChemicalMapper()
        >>> cm.check_synonym_dict(term='dodecene', format_name='main_accession')
        ['HMDB0000933', 'HMDB0059874']

        """
        synonyms = self.database[['synonyms', format_name]].copy()
        synonyms.dropna(how='any', inplace=True)
        synonyms['synonyms'] = synonyms['synonyms'].str.lower()

        hits = synonyms[synonyms['synonyms'].str.contains(term.lower())]
        matches = sorted(set(hits[format_name].values))
        return matches

    def print_info(self):
        """ print information about the dataframe

        Returns
        -------

        """
        print('Number of HMDB accessions = {0}'.format(
            len(self.database['accession'].unique())))
        print('Number of unique KEGG ids = {0}'.format(
            len(self.hmdb_to_kegg.keys())))
        print('Number of HMDB to KEGG mappings = {0}'.format(
            len(self.kegg_to_hmdb.values())))

    def convert_kegg_nodes(self, network):
        """
        Maps network from kegg to gene names

        Parameters
        ----------
        network : nx.DiGraph

        Returns
        -------
        dict

        """

        still_unknown = []
        hits = [i for i in set(network.nodes) if i.startswith('cpd:')]
        net_kegg_names = dict()
        net_chem_names = dict()
        net_cpd_to_hmdb = dict()
        for i in hits:

            name_stripped = i.lstrip('cpd:')
            net_kegg_names[i] = name_stripped

            if name_stripped in self.kegg_to_hmdb:
                mapping = self.kegg_to_hmdb[name_stripped]
                if isinstance(mapping, (list, set, SortedSet)):
                    names = '|'.join(set(mapping))
                    chem_names = set()
                    for name in mapping:
                        try:
                            chem_names.update(self.hmdb_to_chem_name[name])
                        except:
                            continue
                    net_cpd_to_hmdb[i] = names
                    net_chem_names[i] = order_merge(chem_names)

                elif isinstance(mapping, basestring):

                    chem_n = self.hmdb_to_chem_name[mapping]
                    net_cpd_to_hmdb[i] = mapping
                    net_chem_names[i] = '|'.join(chem_n.encode('ascii',
                                                               'ignore'))
                else:
                    print('Returned something else...', mapping)

            elif i in compound_manual:
                loc = compound_manual[i]
                net_cpd_to_hmdb[i] = loc
                if loc in self.hmdb_to_chem_name:
                    net_chem_names[i] = order_merge(
                        self.hmdb_to_chem_name[loc])
            else:
                still_unknown.append(i)
        if len(still_unknown):

            for i in still_unknown:
                name_stripped = i.lstrip('cpd:')
                if name_stripped in self.kegg_hmdb:
                    net_cpd_to_hmdb[i] = self.kegg_hmdb[name_stripped]
                # else:
                #     print("Cannot find a HMDB mapping for %s " % i)
        return net_cpd_to_hmdb, net_kegg_names, net_chem_names


# manually created based on missing in KEGG
compound_manual = {
    'cpd:C07909': 'HMDB0015015',
    'cpd:C16844': 'HMDB0001039',
    'cpd:C00076': 'HMDB0000464',
    'cpd:C00154': 'HMDB0001338',
    'cpd:C01561': 'HMDB0003550',
    'cpd:C04043': 'HMDB0003791',
    'cpd:C01165': 'HMDB0002104',
    'cpd:C00025': 'HMDB0000148',
    'cpd:C00696': 'HMDB0001403',
    'cpd:C00124': 'HMDB0000143',
}


def order_merge(species_set):
    return '|'.join(sorted(species_set))


def tidy_split(df, column, sep='|', keep=False):
    """
    Split the values of a column and expand so the new DataFrame has one split
    value per row. Filters rows where the column is missing.

    Params
    ------
    df : pandas.DataFrame
        dataframe with the column to split and expand
    column : str
        the column to split and expand
    sep : str
        the string used to split the column's values
    keep : bool
        whether to retain the presplit value as it's own row

    Returns
    -------
    pandas.DataFrame
        Returns a dataframe with the same columns as `df`.
    """
    indexes = list()
    new_values = list()
    df = df.dropna(subset=[column])
    for i, presplit in enumerate(df[column].astype(str)):
        values = presplit.split(sep)
        if keep and len(values) > 1:
            indexes.append(i)
            new_values.append(presplit)
        for value in values:
            indexes.append(i)
            new_values.append(value)
    new_df = df.iloc[indexes, :].copy()
    new_df[column] = new_values
    return new_df


if __name__ == "__main__":
    cm = ChemicalMapper()
    print(cm.check_synonym_dict(term='dodecene', format_name='main_accession'))
    print(cm.hmdb_accession_to_main['HMDB15015'])
    print(cm.hmdb_accession_to_main['HMDB0015015'])
    # print(cm.hmdb_to_kegg['HMDB0015015'])
    print(cm.kegg_to_hmdb.keys())
    print(cm.kegg_to_hmdb['C07467'])



import logging
import os

import networkx as nx
import pandas as pd

from magine.data.storage import network_data_dir
from magine.logging import get_logger
from magine.mappings.chemical_mapper import ChemicalMapper
import magine.networks.utils as utils

logger = get_logger(__name__, log_level=logging.INFO)
p_name = os.path.join(network_data_dir, 'biogrid.p.gz')
_base_url = 'https://thebiogrid.org/downloads/archives/Latest%20Release/'
_chem_url = _base_url + 'BIOGRID-CHEMICALS-LATEST.chemtab.zip'
_protein_url = _base_url + 'BIOGRID-ALL-LATEST.tab2.zip'


class BioGridDownload(object):
    def __init__(self):

        self.url = _protein_url
        self.url2 = _chem_url
        self._db_name = 'BioGrid'
        self._cm = ChemicalMapper()

    def _create_chemical_network(self):
        df = pd.read_csv(self.url2,
                         compression='zip',
                         delimiter='\t',
                         error_bad_lines=False,
                         low_memory=False,
                         encoding='utf8',
                         )

        df = df[df['Organism'] == 'Homo sapiens']

        chem_cols = ['Official Symbol',
                     'Action',
                     'Chemical Name',
                     'ATC Codes',
                     'CAS Number',
                     'Pubmed ID',
                     'Chemical Type',
                     'Chemical Source',
                     'Chemical Source ID',
                     'Interaction Type'
                     ]

        df = df[chem_cols]
        df = df[~df['Action'].isin(['unknown'])]

        df.drop_duplicates(
            subset=['Official Symbol', 'Action', 'Chemical Name'],
            inplace=True
        )

        def convert_to_name(row):
            db = row['Chemical Source']
            id_chem_source = row['Chemical Source ID']
            c_name = row['chemName']
            if db == 'DRUGBANK':
                if id_chem_source in self._cm.drugbank_to_hmdb:
                    return sorted(self._cm.drugbank_to_hmdb[id_chem_source])[0]
                elif c_name in self._cm.chem_name_to_hmdb:
                    return sorted(self._cm.chem_name_to_hmdb[c_name])[0]
            return c_name

        def convert_to_hmdb_only(row):
            db = row['Chemical Source']
            id_chem_source = row['Chemical Source ID']
            c_name = row['chemName']
            if db == 'DRUGBANK':
                if id_chem_source in self._cm.drugbank_to_hmdb:
                    return sorted(self._cm.drugbank_to_hmdb[id_chem_source])[0]
                elif c_name in self._cm.chem_name_to_hmdb:
                    return sorted(self._cm.chem_name_to_hmdb[c_name])[0]
            return None

        df['databaseSource'] = self._db_name
        df['pubmedId'] = df['Pubmed ID'].astype(str)

        # cleanup names
        df.rename(columns={'Chemical Type': 'chemType',
                           'Official Symbol': 'gene',
                           'Action': 'interactionType'},
                  inplace=True)

        # keep the same info as other databases (store as compound)
        df.loc[df['chemType'] == 'small molecule', 'chemType'] = 'compound'

        # converting to ascii so we can export play with networkx
        df['chemName'] = df['Chemical Name'].astype(str)

        # convert names to HMBD, or keep it the same if HMDB doesnt exist
        df['target'] = df.apply(convert_to_name, axis=1)

        # add HMDB attribute if it exists
        df['hmdbID'] = df.apply(convert_to_hmdb_only, axis=1).astype(str)

        # create network
        chem_g = nx.from_pandas_edgelist(
            df,
            'gene',
            'target',
            edge_attr=['interactionType', 'databaseSource', 'pubmedId'],
            create_using=nx.DiGraph()
        )
        # df.to_csv('biogrid.csv')
        cols = ['gene', 'target', 'chemName', 'chemType', 'hmdbID']
        chem_table = df[cols].values

        nodes_added = set()

        def add_node(node, node_type, chem_name=None, hmdb=None):
            attr = dict()
            attr['speciesType'] = node_type
            attr['databaseSource'] = self._db_name
            if chem_name is not None:
                attr['chemName'] = chem_name
            if hmdb is not None:
                attr['hmdbNames'] = hmdb

            chem_g.add_node(node, **attr)
            nodes_added.add(node)

        # add node names/attributes
        for row in chem_table:
            gene = row[0]
            chemical = row[1]
            chemical_name = row[2]
            chem_typed = row[3]
            hmdb_id = row[4]
            if gene not in nodes_added:
                add_node(gene, 'gene')

            if chemical not in nodes_added:
                add_node(chemical, chem_typed, chemical_name, hmdb_id)
        return chem_g

    def parse_network(self):
        """
        Parses tab delimited file to networkx.DiGraph


        Returns
        -------

        """
        logger.info("Downloading BioGrid")
        table = pd.read_csv(self.url,
                            compression='zip',
                            delimiter='\t',
                            encoding='utf8',
                            error_bad_lines=False,
                            low_memory=False)
        # only keep human
        # TODO enable other organisms
        table = table.loc[table['Organism Interactor A'].isin(['9606'])]
        table = table.loc[table['Organism Interactor B'].isin(['9606'])]

        protein_cols = ['Official Symbol Interactor A',
                        'Official Symbol Interactor B',
                        'Modification',
                        'Pubmed ID',
                        'Source Database'
                        ]

        table = table[protein_cols].copy()
        # Remove puring binding and no modification
        table = table[~table['Modification'].isin(['-', 'No Modification'])]

        # clean up names
        table['interactionType'] = table['Modification'].str.lower()
        table['pubmedId'] = table['Pubmed ID'].astype(str)

        # cleanup names
        table.rename(columns={'Official Symbol Interactor A': 'source',
                              'Official Symbol Interactor B': 'target',
                              'Source Database': 'databaseSource'},
                     inplace=True)

        # create graph
        protein_graph = nx.from_pandas_edgelist(
            table,
            'source',
            'target',
            edge_attr=['interactionType', 'databaseSource', 'pubmedId'],
            create_using=nx.DiGraph()
        )
        # add names to graph
        nodes = set(table['source'].values).union(set(table['target'].values))
        for node in nodes:
            protein_graph.add_node(node, databaseSource=self._db_name,
                                   speciesType='gene')

        final_graph = utils.compose(protein_graph,
                                    self._create_chemical_network())
        nx.write_gpickle(final_graph, p_name)
        logger.info("Done downloading BioGrid")
        return final_graph


def download_biogrid():
    BioGridDownload().parse_network()


def load_biogrid_network(fresh_download=False):
    """

    Parameters
    ----------
    fresh_download : bool
        Download a fresh copy from biogrid

    Returns
    -------
    nx.DiGraph
    """
    if not os.path.exists(p_name) or fresh_download:
        download_biogrid()

    g = nx.read_gpickle(p_name)
    nn, ne = len(g.nodes()), len(g.edges())
    logger.info("BIOGRID: {} nodes and {} edges".format(nn, ne))
    return g


if __name__ == '__main__':
    BioGridDownload().parse_network()



from magine.networks.network_generator import build_network


def test_build_network():
    build_network(['BAX', 'TP53', 'JAK1', 'BAD', 'HMDB0042489', 'HMDB0059874'],
                  save_name=None, species='hsa',
                  all_measured_list=['CASP3', 'EGFR'])


if __name__ == '__main__':
    test_build_network()



"""
import textwrap

import networkx as nx

import magine.enrichment.ontology_tools as ot
from magine.networks.network_tools import export_to_dot

example_cisplatin = ['GO:0097062', 'GO:0033523', 'GO:0031441', 'GO:1900364',
                     'GO:1900363', 'GO:0022027', 'GO:0090557', 'GO:1901077',
                     'GO:1903299', 'GO:0050686', 'GO:0031468', 'GO:0010666',
                     'GO:0010663', 'GO:0007084', 'GO:0048025', 'GO:0002190',
                     'GO:0046931', 'GO:1903044', 'GO:0035376', 'GO:0006999',
                     'GO:0022027', 'GO:0042776', 'GO:0051292', 'GO:0032594']
topolc_example = ['GO:0044260', 'GO:0006139']


def test_path_to_root():
    go_example = 'GO:0044260'
    g = ot.path_to_root(go_example)
    nodes = set(g.nodes())
    root_nodes = {"GO:0008152",
                  "GO:0043170",
                  "GO:0044260",
                  "GO:0008150",
                  "GO:0071704",
                  "GO:0009987",
                  "GO:0044237"}
    assert nodes == root_nodes


def test_find_common_ancestor():
    topolc_example = ['GO:0044260', 'GO:0006139']
    common_graph = ot.find_disjunction_common_ancestor(topolc_example)
    n_nodes = len(common_graph.nodes())
    print(n_nodes)
    # assert n_nodes == 12
    pretty_format(common_graph, 'go_find_common')


def test_find_common_ancestor_3_terms():
    # This is for the case where one of the terms is in the path to root of
    # one of the other terms
    three_term = ['GO:0006893', 'GO:0031577', 'GO:0033523', 'GO:0010923',
                  'GO:0032786', 'GO:1904667', 'GO:0007094', 'GO:0010828']
    # common_graph = ot.find_disjunction_common_ancestor(topolc_example)
    common_graph = ot.find_disjunction_common_ancestor(three_term)
    # n_nodes = len(common_graph.nodes())
    export_to_dot(common_graph, 'cisplatin_example')
    # assert n_nodes == 12


def test_create_graph_from_list():
    x = ot.create_graph_to_root_from_list_terms(example_cisplatin)
    export_to_dot(x, 'cisplatin_example')

def test_only_one_common():
    # This is for the case where one of the terms is in the path to root of
    # one of the other terms
    common_graph = ot.find_disjunction_common_ancestor(topolc_example)
    # n_nodes = len(common_graph.nodes())
    export_to_dot(common_graph, 'single_ancestor')
    # assert n_nodes == 12


def test_hard_example():
    # Test path that doesn't overlap
    list_of_terms = ['GO:0042776', 'GO:0022027', 'GO:0006999', 'GO:1903044',
                     'GO:0051292']
    list_of_terms = ot.check_term_list(topolc_example)

    common_graph = ot.find_disjunction_common_ancestor(list_of_terms)
    export_to_dot(common_graph, 'common_graph_test')
    ic_dict = nx.get_node_attributes(common_graph, 'ic')
    GOname = nx.get_node_attributes(common_graph, 'GOname')
    new_label = dict()
    for i in ic_dict:
        print(i, ic_dict[i], GOname[i])
        new_label[i] = textwrap.fill("{}\n{}".format(GOname[i], ic_dict[i]),
                                     20)
    nx.set_node_attributes(common_graph, 'label', new_label)
    export_to_dot(common_graph, 'common_graph')


def test_get_common_ancestors():
    list_of_terms = ['GO:0042776', 'GO:0022027', 'GO:0006999', 'GO:1903044',
                     'GO:0051292']
    x = ot.common_parent_go_ids(topolc_example)
    for i in x:
        print(i)


def cisplatin_example():
    common_graph = ot.create_graph_to_root_from_list_terms(example_cisplatin)
    print("Number of nodes = {}".format(len(common_graph.nodes())))
    export_to_dot(common_graph, 'cisplatin_example')

    pretty_format(common_graph, 'cisplatin_all_to_top')

    list_of_terms = ot.check_term_list(example_cisplatin)
    common_graph = ot.find_disjunction_common_ancestor(list_of_terms)
    print("Number of nodes = {}".format(len(common_graph.nodes())))

    export_to_dot(common_graph, 'common_graph_test')
    pretty_format(common_graph, 'cisplatin_term_list_check')


def pretty_format(graph, save_name):
    ic = nx.get_node_attributes(graph, 'ic')
    go_name = nx.get_node_attributes(graph, 'GOname')
    new_label = dict()
    for i in ic:
        new_label[i] = textwrap.fill("{}\n{}".format(go_name[i], ic[i]), 20)

    nx.set_node_attributes(graph, 'label', new_label)
    export_to_dot(graph, save_name)


def slimmed_cisplatin_example():
    example_cisplatin = [
        'GO:0031441',
        'GO:0048025',
        'GO:0050686',
        'GO:1900363',
    ]

    common_graph = ot.create_graph_to_root_from_list_terms(example_cisplatin)
    pretty_format(common_graph, 'cisplatin_rna_example_path_to_top')

    list_of_terms = ot.check_term_list(example_cisplatin)
    common_graph = ot.create_graph_to_root_from_list_terms(example_cisplatin)
    pretty_format(common_graph, 'cisplatin_rna_example_checked_list')

    common_graph = ot.find_disjunction_common_ancestor(list_of_terms)
    pretty_format(common_graph, 'cisplatin_rna_example_checked_list_compress')


if __name__ == "__main__":
    # test_path_to_root()
    # test_find_common_ancestor()
    # test_get_common_ancestors()
    # test_hard_example()
    cisplatin_example()
    # slimmed_cisplatin_example()
    # test_create_graph_from_list()
    # test_find_common_ancestor_3_terms()
    # cisplatin_example()
"""
